@@ -18,7 +18,7 @@
 
 
 from hyperspy._signals.common_signal1d import CommonSignal1D
-from hyperspy._signals.complex_signal import ComplexSignal
+from hyperspy._signals.complex_signal import (ComplexSignal, LazyComplexSignal)
 
 
 class ComplexSignal1D(ComplexSignal, CommonSignal1D):
@@ -32,3 +32,16 @@ class ComplexSignal1D(ComplexSignal, CommonSignal1D):
         if self.axes_manager.signal_dimension != 1:
             self.axes_manager.set_signal_dimension(1)
         self.metadata.Signal.binned = False
+
+
+class LazyComplexSignal1D(LazyComplexSignal, CommonSignal1D):
+
+    """BaseSignal subclass for lazy complex 1-dimensional data."""
+
+    _signal_dimension = 1
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        if self.axes_manager.signal_dimension != 1:
+            self.axes_manager.set_signal_dimension(1)
+        self.metadata.Signal.binned = False
