@@ -19,17 +19,17 @@
 
 import traits.api as t
 
-from hyperspy._signals.eds import EDSSpectrum
+from hyperspy._signals.eds import (EDSSpectrum, LazyEDSSpectrum)
 from hyperspy.defaults_parser import preferences
 from hyperspy.decorators import only_interactive
 
 
-class EDSSEMSpectrum(EDSSpectrum):
+class EDSSEM_mixin:
 
     _signal_type = "EDS_SEM"
 
     def __init__(self, *args, **kwards):
-        EDSSpectrum.__init__(self, *args, **kwards)
+        super().__init__(*args, **kwards)
         # Attributes defaults
         if 'Acquisition_instrument.SEM.Detector.EDS' not in self.metadata:
             if 'Acquisition_instrument.TEM' in self.metadata:
@@ -304,3 +304,11 @@ class EDSSEMSpectrum(EDSSpectrum):
                             auto_add_lines=auto_add_lines,
                             *args, **kwargs)
         return model
+
+
+class EDSSEMSpectrum(EDSSEM_mixin, EDSSpectrum):
+    pass
+
+
+class LazyEDSSEMSpectrum(EDSSEM_mixin, LazyEDSSpectrum):
+    pass
