@@ -16,8 +16,10 @@
 # You should have received a copy of the GNU General Public License
 # along with  HyperSpy.  If not, see <http://www.gnu.org/licenses/>.
 
+import logging
 import matplotlib.pyplot as plt
 import numpy as np
+import dask.array as da
 
 from hyperspy.signal import BaseSignal
 from hyperspy._signals.common_signal1d import CommonSignal1D
@@ -37,8 +39,10 @@ try:
 except:
     statsmodels_installed = False
 
+from hyperspy.misc.utils import stack
 from hyperspy.defaults_parser import preferences
 from hyperspy.external.progressbar import progressbar
+from hyperspy._signals.lazy import lazyerror
 from hyperspy.gui.tools import (
     Signal1DCalibration,
     SmoothingSavitzkyGolay,
@@ -52,6 +56,9 @@ from hyperspy.decorators import interactive_range_selector
 from scipy.ndimage.filters import gaussian_filter1d
 from hyperspy.gui.tools import IntegrateArea
 from hyperspy import components1d
+from hyperspy._signals.lazy import LazySignal
+
+_logger = logging.getLogger(__name__)
 
 
 def find_peaks_ohaver(y, x=None, slope_thresh=0., amp_thresh=None,
@@ -98,17 +105,10 @@ def find_peaks_ohaver(y, x=None, slope_thresh=0., amp_thresh=None,
         contains position, height, and width of each peak
     Examples
     --------
-    >>> x = arange(0,50,0.01)
-    >>> y = cos(x)
-    >>> one_dim_findpeaks(y, x,0,0)
-    array([[  1.68144859e-05,   9.99999943e-01,   3.57487961e+00],
-           [  6.28318614e+00,   1.00000003e+00,   3.57589018e+00],
-           [  1.25663708e+01,   1.00000002e+00,   3.57600673e+00],
-           [  1.88495565e+01,   1.00000002e+00,   3.57597295e+00],
-           [  2.51327421e+01,   1.00000003e+00,   3.57590284e+00],
-           [  3.14159267e+01,   1.00000002e+00,   3.57600856e+00],
-           [  3.76991124e+01,   1.00000002e+00,   3.57597984e+00],
-           [  4.39822980e+01,   1.00000002e+00,   3.57591479e+00]])
+    >>> x = np.arange(0,50,0.01)
+    >>> y = np.cos(x)
+    >>> peaks = find_peaks_ohaver(y, x, 0, 0)
+
     Notes
     -----
     Original code from T. C. O'Haver, 1995.
@@ -172,8 +172,10 @@ def find_peaks_ohaver(y, x=None, slope_thresh=0., amp_thresh=None,
                         c1 = coef[2]
                         c2 = coef[1]
                         c3 = coef[0]
-                        width = np.linalg.norm(
-                            stdev * 2.35703 / (np.sqrt(2) * np.sqrt(-1 * c3)))
+                        with np.errstate(invalid='ignore'):
+                            width = np.linalg.norm(stdev * 2.35703 /
+                                                   (np.sqrt(2) * np.sqrt(-1 *
+                                                                         c3)))
                         # if the peak is too narrow for least-squares
                         # technique to work  well, just use the max value
                         # of y in the sub-group of points near peak.
@@ -218,6 +220,42 @@ def interpolate1D(number_of_interpolation_points, data):
     return interpolator(new_ax)
 
 
+def _estimate_shift1D(data, **kwargs):
+    mask = kwargs.get('mask', None)
+    ref = kwargs.get('ref', None)
+    interpolate = kwargs.get('interpolate', True)
+    ip = kwargs.get('ip', 5)
+    data_slice = kwargs.get('data_slice', slice(None))
+    if bool(mask):
+        return np.nan
+    data = data[data_slice]
+    if interpolate is True:
+        data = interpolate1D(ip, data)
+    return np.argmax(np.correlate(ref, data, 'full')) - len(ref) + 1
+
+
+def _shift1D(data, **kwargs):
+    shift = kwargs.get('shift', 0.)
+    original_axis = kwargs.get('original_axis', None)
+    fill_value = kwargs.get('fill_value', np.nan)
+    kind = kwargs.get('kind', 'linear')
+    offset = kwargs.get('offset', 0.)
+    scale = kwargs.get('scale', 1.)
+    size = kwargs.get('size', 2)
+    if np.isnan(shift):
+        return data
+    axis = np.linspace(offset, offset + scale * (size - 1), size)
+
+    si = sp.interpolate.interp1d(original_axis,
+                                 data,
+                                 bounds_error=False,
+                                 fill_value=fill_value,
+                                 kind=kind)
+    offset = float(offset - shift)
+    axis = np.linspace(offset, offset + scale * (size - 1), size)
+    return si(axis)
+
+
 class Signal1D(BaseSignal, CommonSignal1D):
 
     """
@@ -328,6 +366,7 @@ class Signal1D(BaseSignal, CommonSignal1D):
                 crop=True,
                 expand=False,
                 fill_value=np.nan,
+                parallel=None,
                 show_progressbar=None):
         """Shift the data in place over the signal axis by the amount specified
         by an array.
@@ -350,6 +389,7 @@ class Signal1D(BaseSignal, CommonSignal1D):
         fill_value : float
             If crop is False fill the data outside of the original
             interval with the given value where needed.
+        parallel : {None, bool}
         show_progressbar : None or bool
             If True, display a progress bar. If None the default is set in
             `preferences`.
@@ -379,37 +419,55 @@ class Signal1D(BaseSignal, CommonSignal1D):
         else:
             ilow = axis.low_index
         if expand:
-            padding = []
-            for i in range(self.data.ndim):
-                if i == axis.index_in_array:
-                    padding.append(
-                        (axis.high_index - ihigh + 1, ilow - axis.low_index))
-                else:
-                    padding.append((0, 0))
-            self.data = np.pad(self.data, padding, mode='constant',
-                               constant_values=(fill_value,))
+            if self._lazy:
+                ind = axis.index_in_array
+                pre_shape = list(self.data.shape)
+                post_shape = list(self.data.shape)
+                pre_chunks = list(self.data.chunks)
+                post_chunks = list(self.data.chunks)
+
+                pre_shape[ind] = axis.high_index - ihigh + 1
+                post_shape[ind] = ilow - axis.low_index
+                for chunks, shape in zip((pre_chunks, post_chunks),
+                                         (pre_shape, post_shape)):
+                    maxsize = min(np.max(chunks[ind]), shape[ind])
+                    num = np.ceil(shape[ind] / maxsize)
+                    chunks[ind] = tuple(len(ar) for ar in
+                                        np.array_split(np.arange(shape[ind]),
+                                                       num))
+                pre_array = da.full(tuple(pre_shape),
+                                    fill_value,
+                                    chunks=tuple(pre_chunks))
+
+                post_array = da.full(tuple(post_shape),
+                                     fill_value,
+                                     chunks=tuple(post_chunks))
+
+                self.data = da.concatenate((pre_array, self.data, post_array),
+                                           axis=ind)
+            else:
+                padding = []
+                for i in range(self.data.ndim):
+                    if i == axis.index_in_array:
+                        padding.append((axis.high_index - ihigh + 1,
+                                        ilow - axis.low_index))
+                    else:
+                        padding.append((0, 0))
+                self.data = np.pad(self.data, padding, mode='constant',
+                                   constant_values=(fill_value,))
             axis.offset += minimum
             axis.size += axis.high_index - ihigh + 1 + ilow - axis.low_index
-        offset = axis.offset
-        original_axis = axis.axis.copy()
-        with progressbar(total=self.axes_manager.navigation_size,
-                         disable=not show_progressbar,
-                         leave=True) as pbar:
-            for i, (dat, shift) in enumerate(zip(
-                    self._iterate_signal(),
-                    shift_array.ravel())):
-                if np.isnan(shift):
-                    continue
-                si = sp.interpolate.interp1d(original_axis,
-                                             dat,
-                                             bounds_error=False,
-                                             fill_value=fill_value,
-                                             kind=interpolation_method)
-                axis.offset = float(offset - shift)
-                dat[:] = si(axis.axis)
-                pbar.update(1)
 
-        axis.offset = offset
+        self._map_iterate(_shift1D, (('shift', shift_array.ravel()),),
+                          original_axis=axis.axis,
+                          fill_value=fill_value,
+                          kind=interpolation_method,
+                          offset=axis.offset,
+                          scale=axis.scale,
+                          size=axis.size,
+                          show_progressbar=show_progressbar,
+                          parallel=parallel,
+                          ragged=False)
 
         if crop and not expand:
             self.crop(axis.index_in_axes_manager,
@@ -418,7 +476,7 @@ class Signal1D(BaseSignal, CommonSignal1D):
 
         self.events.data_changed.trigger(obj=self)
 
-    def interpolate_in_between(self, start, end, delta=3,
+    def interpolate_in_between(self, start, end, delta=3, parallel=None,
                                show_progressbar=None, **kwargs):
         """Replace the data in a given range by interpolation.
         The operation is performed in place.
@@ -432,6 +490,7 @@ class Signal1D(BaseSignal, CommonSignal1D):
         show_progressbar : None or bool
             If True, display a progress bar. If None the default is set in
             `preferences`.
+        parallel: {None, bool}
         All extra keyword arguments are passed to
         scipy.interpolate.interp1d. See the function documentation
         for details.
@@ -449,16 +508,16 @@ class Signal1D(BaseSignal, CommonSignal1D):
             delta = int(delta / axis.scale)
         i0 = int(np.clip(i1 - delta, 0, np.inf))
         i3 = int(np.clip(i2 + delta, 0, axis.size))
-        with progressbar(total=self.axes_manager.navigation_size,
-                         disable=not show_progressbar,
-                         leave=True) as pbar:
-            for i, dat in enumerate(self._iterate_signal()):
-                dat_int = sp.interpolate.interp1d(
-                    list(range(i0, i1)) + list(range(i2, i3)),
-                    dat[i0:i1].tolist() + dat[i2:i3].tolist(),
-                    **kwargs)
-                dat[i1:i2] = dat_int(list(range(i1, i2)))
-                pbar.update(1)
+
+        def interpolating_function(dat):
+            dat_int = sp.interpolate.interp1d(
+                list(range(i0, i1)) + list(range(i2, i3)),
+                dat[i0:i1].tolist() + dat[i2:i3].tolist(),
+                **kwargs)
+            dat[i1:i2] = dat_int(list(range(i1, i2)))
+            return dat
+        self._map_iterate(interpolating_function, ragged=False,
+                          parallel=parallel, show_progressbar=show_progressbar)
         self.events.data_changed.trigger(obj=self)
 
     def _check_navigation_mask(self, mask):
@@ -481,6 +540,7 @@ class Signal1D(BaseSignal, CommonSignal1D):
                          interpolate=True,
                          number_of_interpolation_points=5,
                          mask=None,
+                         parallel=None,
                          show_progressbar=None):
         """Estimate the shifts in the current signal axis using
          cross-correlation.
@@ -512,6 +572,7 @@ class Signal1D(BaseSignal, CommonSignal1D):
             It must have signal_dimension = 0 and navigation_shape equal to the
             current signal. Where mask is True the shift is not computed
             and set to nan.
+        parallel : {None, bool}
         show_progressbar : None or bool
             If True, display a progress bar. If None the default is set in
             `preferences`.
@@ -528,37 +589,40 @@ class Signal1D(BaseSignal, CommonSignal1D):
         ip = number_of_interpolation_points + 1
         axis = self.axes_manager.signal_axes[0]
         self._check_navigation_mask(mask)
+        # we compute for now
+        if isinstance(start, da.Array):
+            start = start.compute()
+        if isinstance(end, da.Array):
+            end = end.compute()
+        i1, i2 = axis._get_index(start), axis._get_index(end)
         if reference_indices is None:
             reference_indices = self.axes_manager.indices
-
-        i1, i2 = axis._get_index(start), axis._get_index(end)
-        shift_array = np.zeros(self.axes_manager._navigation_shape_in_array,
-                               dtype=float)
         ref = self.inav[reference_indices].data[i1:i2]
+
         if interpolate is True:
             ref = interpolate1D(ip, ref)
-        with progressbar(total=self.axes_manager.navigation_size,
-                         disable=not show_progressbar,
-                         leave=True) as pbar:
-            for i, (dat, indices) in enumerate(zip(
-                    self._iterate_signal(),
-                    self.axes_manager._array_indices_generator())):
-                if mask is not None and bool(mask.data[indices]) is True:
-                    shift_array[indices] = np.nan
-                else:
-                    dat = dat[i1:i2]
-                    if interpolate is True:
-                        dat = interpolate1D(ip, dat)
-                    shift_array[indices] = np.argmax(
-                        np.correlate(ref, dat, 'full')) - len(ref) + 1
-                pbar.update(1)
-
+        iterating_kwargs = ()
+        if mask is not None:
+            iterating_kwargs += (('mask', mask),)
+        shift_signal = self._map_iterate(
+            _estimate_shift1D,
+            iterating_kwargs=iterating_kwargs,
+            data_slice=slice(i1, i2),
+            mask=None,
+            ref=ref,
+            ip=ip,
+            interpolate=interpolate,
+            ragged=False,
+            parallel=parallel,
+            inplace=False,
+            show_progressbar=show_progressbar,)
+        shift_array = shift_signal.data
         if max_shift is not None:
             if interpolate is True:
                 max_shift *= ip
             shift_array.clip(-max_shift, max_shift)
         if interpolate is True:
-            shift_array /= ip
+            shift_array = shift_array / ip
         shift_array *= axis.scale
         return shift_array
 
@@ -573,7 +637,7 @@ class Signal1D(BaseSignal, CommonSignal1D):
                 crop=True,
                 expand=False,
                 fill_value=np.nan,
-                also_align=[],
+                also_align=None,
                 mask=None,
                 show_progressbar=None):
         """Estimate the shifts in the signal axis using
@@ -616,7 +680,7 @@ class Signal1D(BaseSignal, CommonSignal1D):
         fill_value : float
             If crop is False fill the data outside of the original
             interval with the given value where needed.
-        also_align : list of signals
+        also_align : list of signals, None
             A list of BaseSignal instances that has exactly the same
             dimensions as this one and that will be aligned using the shift map
             estimated using the this signal.
@@ -641,6 +705,13 @@ class Signal1D(BaseSignal, CommonSignal1D):
         if also_align is None:
             also_align = []
         self._check_signal_dimension_equals_one()
+        if self._lazy:
+            _logger.warning('In order to properly expand, the lazy '
+                            'reference signal will be read twice (once to '
+                            'estimate shifts, and second time to shift '
+                            'appropriatelly), which might take a long time. '
+                            'Use expand=False to only pass through the data '
+                            'once.')
         shift_array = self.estimate_shift1D(
             start=start,
             end=end,
@@ -650,7 +721,8 @@ class Signal1D(BaseSignal, CommonSignal1D):
             number_of_interpolation_points=number_of_interpolation_points,
             mask=mask,
             show_progressbar=show_progressbar)
-        for signal in also_align + [self]:
+        signals_to_shift = [self] + also_align
+        for signal in signals_to_shift:
             signal.shift1D(shift_array=shift_array,
                            interpolation_method=interpolation_method,
                            crop=crop,
@@ -681,12 +753,16 @@ class Signal1D(BaseSignal, CommonSignal1D):
         Examples
         --------
         Using the GUI
-        >>> s.integrate_in_range()
+        >>> s = hs.signals.Signal1D(range(1000))
+        >>> s.integrate_in_range() #doctest: +SKIP
+
         Using the CLI
         >>> s_int = s.integrate_in_range(signal_range=(560,None))
+
         Selecting a range in the axis units, by specifying the
         signal range with floats.
         >>> s_int = s.integrate_in_range(signal_range=(560.,590.))
+
         Selecting a range using the index, by specifying the
         signal range with integers.
         >>> s_int = s.integrate_in_range(signal_range=(100,120))
@@ -732,7 +808,8 @@ class Signal1D(BaseSignal, CommonSignal1D):
     def smooth_savitzky_golay(self,
                               polynomial_order=None,
                               window_length=None,
-                              differential_order=0):
+                              differential_order=0,
+                              parallel=None):
         """Apply a Savitzky-Golay filter to the data in place.
         If `polynomial_order` or `window_length` or `differential_order` are
         None the method is run in interactive mode.
@@ -748,6 +825,8 @@ class Signal1D(BaseSignal, CommonSignal1D):
             The order of the derivative to compute.  This must be a
             nonnegative integer.  The default is 0, which means to filter
             the data without differentiating.
+        parallel : {bool, None}
+            Perform the operation in a threaded manner (parallely).
         Notes
         -----
         More information about the filter in `scipy.signal.savgol_filter`.
@@ -759,14 +838,9 @@ class Signal1D(BaseSignal, CommonSignal1D):
         if (polynomial_order is not None and
                 window_length is not None):
             axis = self.axes_manager.signal_axes[0]
-            self.data = savgol_filter(
-                x=self.data,
-                window_length=window_length,
-                polyorder=polynomial_order,
-                deriv=differential_order,
-                delta=axis.scale,
-                axis=axis.index_in_array)
-            self.events.data_changed.trigger(obj=self)
+            self.map(savgol_filter, window_length=window_length,
+                     polyorder=polynomial_order, deriv=differential_order,
+                     delta=axis.scale, ragged=False, parallel=parallel)
         else:
             # Interactive mode
             smoother = SmoothingSavitzkyGolay(self)
@@ -780,7 +854,8 @@ class Signal1D(BaseSignal, CommonSignal1D):
     def smooth_lowess(self,
                       smoothing_parameter=None,
                       number_of_iterations=None,
-                      show_progressbar=None):
+                      show_progressbar=None,
+                      parallel=None):
         """Lowess data smoothing in place.
         If `smoothing_parameter` or `number_of_iterations` are None the method
         is run in interactive mode.
@@ -795,6 +870,8 @@ class Signal1D(BaseSignal, CommonSignal1D):
         show_progressbar : None or bool
             If True, display a progress bar. If None the default is set in
             `preferences`.
+        parallel : {Bool, None, int}
+            Perform the operation parallely
         Raises
         ------
         SignalDimensionError if the signal dimension is not 1.
@@ -822,9 +899,12 @@ class Signal1D(BaseSignal, CommonSignal1D):
                      it=number_of_iterations,
                      is_sorted=True,
                      return_sorted=False,
-                     show_progressbar=show_progressbar)
+                     show_progressbar=show_progressbar,
+                     ragged=False,
+                     parallel=parallel)
 
-    def smooth_tv(self, smoothing_parameter=None, show_progressbar=None):
+    def smooth_tv(self, smoothing_parameter=None, show_progressbar=None,
+                  parallel=None):
         """Total variation data smoothing in place.
         Parameters
         ----------
@@ -834,6 +914,8 @@ class Signal1D(BaseSignal, CommonSignal1D):
         show_progressbar : None or bool
             If True, display a progress bar. If None the default is set in
             `preferences`.
+        parallel : {Bool, None, int}
+            Perform the operation parallely
         Raises
         ------
         SignalDimensionError if the signal dimension is not 1.
@@ -844,7 +926,9 @@ class Signal1D(BaseSignal, CommonSignal1D):
             smoother.edit_traits()
         else:
             self.map(_tv_denoise_1d, weight=smoothing_parameter,
-                     show_progressbar=show_progressbar)
+                     ragged=False,
+                     show_progressbar=show_progressbar,
+                     parallel=parallel)
 
     def filter_butterworth(self,
                            cutoff_frequency_ratio=None,
@@ -911,11 +995,15 @@ class Signal1D(BaseSignal, CommonSignal1D):
         Examples
         --------
         Using gui, replaces spectrum s
-        >>>> s.remove_background()
+        >>> s = hs.signals.Signal1D(range(1000))
+        >>> s.remove_background() #doctest: +SKIP
+
         Using command line, returns a spectrum
-        >>>> s = s.remove_background(signal_range=(400,450), background_type='PowerLaw')
+        >>> s1 = s.remove_background(signal_range=(400,450), background_type='PowerLaw')
+
         Using a full model to fit the background
-        >>>> s = s.remove_background(signal_range=(400,450), fast=False)
+        >>> s1 = s.remove_background(signal_range=(400,450), fast=False)
+
         Raises
         ------
         SignalDimensionError if the signal dimension is not 1.
@@ -995,11 +1083,7 @@ class Signal1D(BaseSignal, CommonSignal1D):
                 "FWHM must be greater than zero")
         axis = self.axes_manager.signal_axes[0]
         FWHM *= 1 / axis.scale
-        self.data = gaussian_filter1d(
-            self.data,
-            axis=axis.index_in_array,
-            sigma=FWHM / 2.35482)
-        self.events.data_changed.trigger(obj=self)
+        self.map(gaussian_filter1d, sigma=FWHM / 2.35482, ragged=False)
 
     def hanning_taper(self, side='both', channels=None, offset=0):
         """Apply a hanning taper to the data in place.
@@ -1022,6 +1106,8 @@ class Signal1D(BaseSignal, CommonSignal1D):
 
         """
         # TODO: generalize it
+        if self._lazy:
+            raise lazyerror
         self._check_signal_dimension_equals_one()
         if channels is None:
             channels = int(round(len(self()) * 0.02))
@@ -1046,7 +1132,7 @@ class Signal1D(BaseSignal, CommonSignal1D):
 
     def find_peaks1D_ohaver(self, xdim=None, slope_thresh=0, amp_thresh=None,
                             subchannel=True, medfilt_radius=5, maxpeakn=30000,
-                            peakgroup=10):
+                            peakgroup=10, parallel=None):
         """Find peaks along a 1D line (peaks in spectrum/spectra).
 
         Function to locate the positive peaks in a noisy x-y data set.
@@ -1096,6 +1182,9 @@ class Signal1D(BaseSignal, CommonSignal1D):
         subpix : bool (optional)
                  default is set to True
 
+        parallel : {None, bool}
+            Perform the operation in a threaded (parallel) manner.
+
         Returns
         -------
         peaks : structured array of shape _navigation_shape_in_array in which
@@ -1111,27 +1200,24 @@ class Signal1D(BaseSignal, CommonSignal1D):
         # TODO: add scipy.signal.find_peaks_cwt
         self._check_signal_dimension_equals_one()
         axis = self.axes_manager.signal_axes[0].axis
-        arr_shape = (self.axes_manager._navigation_shape_in_array
-                     if self.axes_manager.navigation_size > 0
-                     else [1, ])
-        peaks = np.zeros(arr_shape, dtype=object)
-        for y, indices in zip(self._iterate_signal(),
-                              self.axes_manager._array_indices_generator()):
-            peaks[indices] = find_peaks_ohaver(
-                y,
-                axis,
-                slope_thresh=slope_thresh,
-                amp_thresh=amp_thresh,
-                medfilt_radius=medfilt_radius,
-                maxpeakn=maxpeakn,
-                peakgroup=peakgroup,
-                subchannel=subchannel)
-        return peaks
+        peaks = self.map(find_peaks_ohaver,
+                         x=axis,
+                         slope_thresh=slope_thresh,
+                         amp_thresh=amp_thresh,
+                         medfilt_radius=medfilt_radius,
+                         maxpeakn=maxpeakn,
+                         peakgroup=peakgroup,
+                         subchannel=subchannel,
+                         ragged=True,
+                         parallel=parallel,
+                         inplace=False)
+        return peaks.data
 
     def estimate_peak_width(self,
                             factor=0.5,
                             window=None,
                             return_interval=False,
+                            parallel=None,
                             show_progressbar=None):
         """Estimate the width of the highest intensity of peak
         of the spectra at a given fraction of its maximum.
@@ -1155,6 +1241,7 @@ class Signal1D(BaseSignal, CommonSignal1D):
             If True, returns 2 extra signals with the positions of the
             desired height fraction at the left and right of the
             peak.
+        parallel : {None, bool}
         show_progressbar : None or bool
             If True, display a progress bar. If None the default is set in
             `preferences`.
@@ -1172,34 +1259,41 @@ class Signal1D(BaseSignal, CommonSignal1D):
         if not 0 < factor < 1:
             raise ValueError("factor must be between 0 and 1.")
 
-        left, right = (self._get_navigation_signal(),
-                       self._get_navigation_signal())
-        # The signals must be of dtype float to contain np.nan
-        left.change_dtype('float')
-        right.change_dtype('float')
         axis = self.axes_manager.signal_axes[0]
-        x = axis.axis
+        # x = axis.axis
         maxval = self.axes_manager.navigation_size
         show_progressbar = show_progressbar and maxval > 0
-        for i, spectrum in progressbar(enumerate(self),
-                                       total=maxval,
-                                       disable=not show_progressbar,
-                                       leave=True):
+
+        def estimating_function(spectrum,
+                                window=None,
+                                factor=0.5,
+                                axis=None):
+            x = axis.axis
             if window is not None:
-                vmax = axis.index2value(spectrum.data.argmax())
-                spectrum = spectrum.isig[vmax - window / 2.:vmax + window / 2.]
-                x = spectrum.axes_manager[0].axis
+                vmax = axis.index2value(spectrum.argmax())
+                slices = axis._get_array_slices(
+                    slice(vmax - window * 0.5, vmax + window * 0.5))
+                spectrum = spectrum[slices]
+                x = x[slices]
             spline = scipy.interpolate.UnivariateSpline(
                 x,
-                spectrum.data - factor * spectrum.data.max(),
+                spectrum - factor * spectrum.max(),
                 s=0)
             roots = spline.roots()
             if len(roots) == 2:
-                left.isig[self.axes_manager.indices] = roots[0]
-                right.isig[self.axes_manager.indices] = roots[1]
+                return np.array(roots)
             else:
-                left.isig[self.axes_manager.indices] = np.nan
-                right.isig[self.axes_manager.indices] = np.nan
+                return np.full((2,), np.nan)
+
+        both = self._map_iterate(estimating_function,
+                                 window=window,
+                                 factor=factor,
+                                 axis=axis,
+                                 ragged=False,
+                                 inplace=False,
+                                 parallel=parallel,
+                                 show_progressbar=show_progressbar)
+        left, right = both.T.split()
         width = right - left
         if factor == 0.5:
             width.metadata.General.title = (
@@ -1227,3 +1321,14 @@ class Signal1D(BaseSignal, CommonSignal1D):
             return [width, left, right]
         else:
             return width
+
+
+class LazySignal1D(LazySignal, Signal1D):
+
+    """
+    """
+    _lazy = True
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.axes_manager.set_signal_dimension(1)
