@@ -262,7 +262,8 @@ class ImagePlot(BlittedFigure):
 
         if self.colorbar is True:
             self._colorbar = plt.colorbar(self.ax.images[0], ax=self.ax)
-            self._colorbar.set_label(self.quantity_label, rotation=-90, va='bottom')
+            self._colorbar.set_label(
+                self.quantity_label, rotation=-90, va='bottom')
             self._colorbar.ax.yaxis.set_animated(True)
 
         self.figure.canvas.draw()
@@ -327,7 +328,7 @@ class ImagePlot(BlittedFigure):
                     self.vmax != self.vmin):
                 redraw_colorbar = True
                 ims[0].autoscale()
-
+        redraw_colorbar = redraw_colorbar and self.colorbar
         if self.plot_indices is True:
             self._text.set_text(self.axes_manager.indices)
         if self.no_nans:
