@@ -47,7 +47,18 @@ class MPL_HyperExplorer(object):
         # Doing nothing is good enough for signal_dimension==0 though.
         return
 
-    def plot_navigator(self):
+    def plot_navigator(self,
+                       colorbar=True,
+                       scalebar=True,
+                       scalebar_color="white",
+                       axes_ticks=None,
+                       saturated_pixels=0,
+                       vmin=None,
+                       vmax=None,
+                       no_nans=False,
+                       centre_colormap="auto",
+                       title=None,
+                       **kwds):
         if self.axes_manager.navigation_dimension == 0:
             return
         if self.navigator_data_function is None:
@@ -57,14 +68,13 @@ class MPL_HyperExplorer(object):
                 self.axes_manager.navigation_axes,
                 title=self.signal_title + " navigation sliders")
             return
+        title = title or self.signal_title + " Navigator" if self.signal_title else ""
         if self.navigator_plot is not None:
             self.navigator_plot.plot()
             return
         elif len(self.navigator_data_function().shape) == 1:
             # Create the figure
-            sf = signal1d.Signal1DFigure(title=self.signal_title + ' Navigator'
-                                         if self.signal_title
-                                         else "")
+            sf = signal1d.Signal1DFigure(title=title)
             axis = self.axes_manager.navigation_axes[0]
             sf.xlabel = '%s' % str(axis)
             if axis.units is not Undefined:
@@ -96,6 +106,15 @@ class MPL_HyperExplorer(object):
         elif len(self.navigator_data_function().shape) >= 2:
             imf = image.ImagePlot()
             imf.data_function = self.navigator_data_function
+            imf.colorbar = colorbar
+            imf.scalebar = scalebar
+            imf.scalebar_color = scalebar_color
+            imf.axes_ticks = axes_ticks
+            imf.saturated_pixels = saturated_pixels
+            imf.vmin = vmin
+            imf.vmax = vmax
+            imf.no_nans = no_nans
+            imf.centre_colormap = centre_colormap
             # Navigator labels
             if self.axes_manager.navigation_dimension == 1:
                 imf.yaxis = self.axes_manager.navigation_axes[0]
@@ -113,8 +132,8 @@ class MPL_HyperExplorer(object):
                             partial(axis.events.index_changed.disconnect,
                                     imf.update), [])
 
-            imf.title = self.signal_title + ' Navigator'
-            imf.plot()
+            imf.title = title
+            imf.plot(**kwds)
             self.pointer.set_mpl_ax(imf.ax)
             self.navigator_plot = imf
 
@@ -132,7 +151,7 @@ class MPL_HyperExplorer(object):
                 self.pointer = pointer(self.axes_manager)
                 self.pointer.color = 'red'
                 self.pointer.connect_navigate()
-            self.plot_navigator()
+            self.plot_navigator(**kwargs.pop('navigator_kwds', {}))
         self.plot_signal(**kwargs)
 
     def assign_pointer(self):
