@@ -42,3 +42,5 @@ elif "Qt" in backend:
     set_ets_toolkit("qt4")
 else:
     set_ets_toolkit("null")
+    from hyperspy.defaults_parser import preferences
+    preferences.General.interactive = False
