@@ -20,7 +20,7 @@ import struct
 import warnings
 from glob import glob
 import os
-from datetime import datetime
+from dateutil import parser
 import logging
 import xml.etree.ElementTree as ET
 try:
@@ -33,7 +33,7 @@ import numpy as np
 import traits.api as t
 
 from hyperspy.misc.array_tools import sarray2dict
-from hyperspy.misc.utils import DictionaryTreeBrowser
+from hyperspy.misc.utils import DictionaryTreeBrowser, multiply
 
 _logger = logging.getLogger(__name__)
 
@@ -273,7 +273,7 @@ def emi_reader(filename, dump_xml=False, **kwds):
     for f in ser_files:
         _logger.info("Opening %s", f)
         try:
-            sers.append(ser_reader(f, objects))
+            sers.append(ser_reader(f, objects, **kwds))
         except IOError:  # Probably a single spectrum that we don't support
             continue
 
@@ -466,44 +466,49 @@ def ser_reader(filename, objects=None, *args, **kwds):
     header, data = load_ser_file(filename)
     record_by = guess_record_by(header['DataTypeID'])
     ndim = int(header['NumberDimensions'])
-    date, time = "", ""
-    if record_by == 'spectrum':
-        if ndim == 0 and header["ValidNumberElements"] != 0:
-            # The calibration of the axes are not stored in the header.
-            # We try to guess from the position coordinates.
-            array_shape, axes = get_axes_from_position(header=header,
-                                                       data=data)
-        else:
-            axes = []
-            array_shape = [None, ] * int(ndim)
-            if len(data['PositionY']) > 1 and \
-                    (data['PositionY'][0] == data['PositionY'][1]):
-                # The spatial dimensions are stored in F order i.e. X, Y, ...
-                order = "F"
-            else:
-                # The spatial dimensions are stored in C order i.e. ..., Y, X
-                order = "C"
-            # Extra dimensions
-            for i in range(ndim):
-                if i == ndim - 1:
-                    name = 'x'
-                elif i == ndim - 2:
-                    name = 'y'
+    date, time = None, None
+    if objects is not None:
+        objects_dict = convert_xml_to_dict(objects[0])
+        date, time = _get_date_time(objects_dict.ObjectInfo.AcquireDate)
+    if "PositionY" in data.dtype.names and len(data['PositionY']) > 1 and \
+            (data['PositionY'][0] == data['PositionY'][1]):
+        # The spatial dimensions are stored in F order i.e. X, Y, ...
+        order = "F"
+    else:
+        # The spatial dimensions are stored in C order i.e. ..., Y, X
+        order = "C"
+    if ndim == 0 and header["ValidNumberElements"] != 0:
+        # The calibration of the axes are not stored in the header.
+        # We try to guess from the position coordinates.
+        array_shape, axes = get_axes_from_position(header=header,
+                                                   data=data)
+    else:
+        axes = []
+        array_shape = [None, ] * int(ndim)
+        spatial_axes = ["x", "y"][:ndim]
+        for i in range(ndim):
+            idim = 1 + i if order == "C" else ndim - i
+            if (record_by == "spectrum" or
+                    header['Dim-%i_DimensionSize' % (i + 1)][0] != 1):
+                units = (header['Dim-%i_Units' % (idim)][0].decode('utf-8')
+                         if header['Dim-%i_UnitsLength' % (idim)] > 0
+                         else t.Undefined)
+                if units == "meters":
+                    name = (spatial_axes.pop() if order == "F"
+                            else spatial_axes.pop(-1))
                 else:
                     name = t.Undefined
-                idim = 1 + i if order == "C" else ndim - i
                 axes.append({
-                    'name': name,
                     'offset': header['Dim-%i_CalibrationOffset' % idim][0],
                     'scale': header['Dim-%i_CalibrationDelta' % idim][0],
-                    'units': header['Dim-%i_Units' % idim][0].decode('utf-8'),
+                    'units': units,
                     'size': header['Dim-%i_DimensionSize' % idim][0],
-                    'index_in_array': i
+                    'name': name,
                 })
                 array_shape[i] = \
                     header['Dim-%i_DimensionSize' % idim][0]
-
-        # Spectral dimension
+    # Spectral dimension
+    if record_by == "spectrum":
         axes.append({
             'offset': data['CalibrationOffset'][0],
             'scale': data['CalibrationDelta'][0],
@@ -519,34 +524,8 @@ def ser_reader(filename, objects=None, *args, **kwds):
         array_shape.append(data['ArrayLength'][0])
 
     elif record_by == 'image':
-        # Extra dimensions
-        if ndim == 0 and header["ValidNumberElements"] != 0:
-            # The calibration of the axes are not stored in the header.
-            # We try to guess from the position coordinates.
-            array_shape, axes = get_axes_from_position(header=header,
-                                                       data=data)
-        else:
-            axes = []
-            array_shape = []
-            for i in range(ndim):
-                if header['Dim-%i_DimensionSize' % (i + 1)][0] != 1:
-                    axes.append({
-                        'offset': header[
-                            'Dim-%i_CalibrationOffset' % (i + 1)][0],
-                        'scale': header[
-                            'Dim-%i_CalibrationDelta' % (i + 1)][0],
-                        # for image stack, the UnitsLength is 0 (no units)
-                        'units': header['Dim-%i_Units' % (i + 1)][0].decode(
-                            'utf-8')
-                        if header['Dim-%i_UnitsLength' % (i + 1)] > 0
-                        else 'Unknown',
-                        'size': header['Dim-%i_DimensionSize' % (i + 1)][0],
-                    })
-                array_shape.append(header['Dim-%i_DimensionSize' % (i + 1)][0])
         if objects is not None:
-            objects_dict = convert_xml_to_dict(objects[0])
             units = _guess_units_from_mode(objects_dict, header)
-            date, time = _get_date_time(objects_dict.ObjectInfo.AcquireDate)
         else:
             units = "meters"
         # Y axis
@@ -579,13 +558,59 @@ def ser_reader(filename, objects=None, *args, **kwds):
         elif axis['units'] == '1/meters':
             axis['units'] = '1/nm'
             axis['scale'] /= 10 ** 9
+    # Remove Nones from array_shape caused by squeezing size 1 dimensions
+    array_shape = [dim for dim in array_shape if dim is not None]
+    lazy = kwds.pop('lazy', False)
+    if lazy:
+        from dask import delayed
+        from dask.array import from_delayed
+        val = delayed(load_only_data, pure=True)(filename, array_shape,
+                                                 record_by, len(axes))
+        dc = from_delayed(val, shape=array_shape,
+                          dtype=data['Array'].dtype)
+    else:
+        dc = load_only_data(filename, array_shape, record_by, len(axes),
+                            data=data)
+
+    if ordict:
+        original_metadata = OrderedDict()
+    else:
+        original_metadata = {}
+    header_parameters = sarray2dict(header)
+    sarray2dict(data, header_parameters)
+    # We remove the Array key to save memory avoiding duplication
+    del header_parameters['Array']
+    original_metadata['ser_header_parameters'] = header_parameters
+    metadata = {'General': {
+        'original_filename': os.path.split(filename)[1],
+    },
+        "Signal": {
+        'signal_type': "",
+        'record_by': record_by,
+    },
+    }
+    if date is not None and time is not None:
+        metadata['General']['date'] = date
+        metadata['General']['time'] = time
+    dictionary = {
+        'data': dc,
+        'metadata': metadata,
+        'axes': axes,
+        'original_metadata': original_metadata,
+        'mapping': mapping}
+    return dictionary
+
+
+def load_only_data(filename, array_shape, record_by, num_axes, data=None):
+    if data is None:
+        _, data = load_ser_file(filename)
     # If the acquisition stops before finishing the job, the stored file will
     # report the requested size even though no values are recorded. Therefore
     # if the shapes of the retrieved array does not match that of the data
     # dimensions we must fill the rest with zeros or (better) nans if the
     # dtype is float
-    if np.cumprod(array_shape)[-1] != np.cumprod(data['Array'].shape)[-1]:
-        dc = np.zeros(np.cumprod(array_shape)[-1],
+    if multiply(array_shape) != multiply(data['Array'].shape):
+        dc = np.zeros(multiply(array_shape),
                       dtype=data['Array'].dtype)
         if dc.dtype is np.dtype('f') or dc.dtype is np.dtype('f8'):
             dc[:] = np.nan
@@ -596,36 +621,11 @@ def ser_reader(filename, objects=None, *args, **kwds):
     dc = dc.reshape(array_shape)
     if record_by == 'image':
         dc = dc[..., ::-1, :]
-    if ordict:
-        original_metadata = OrderedDict()
-    else:
-        original_metadata = {}
-    header_parameters = sarray2dict(header)
-    sarray2dict(data, header_parameters)
-    if len(axes) != len(dc.shape):
+    if num_axes != len(dc.shape):
         dc = dc.squeeze()
-    if len(axes) != len(dc.shape):
+    if num_axes != len(dc.shape):
         raise IOError("Please report this issue to the HyperSpy developers.")
-    # We remove the Array key to save memory avoiding duplication
-    del header_parameters['Array']
-    original_metadata['ser_header_parameters'] = header_parameters
-    dictionary = {
-        'data': dc,
-        'metadata': {
-            'General': {
-                'original_filename': os.path.split(filename)[1],
-                'date': date,
-                'time': time,
-            },
-            "Signal": {
-                'signal_type': "",
-                'record_by': record_by,
-            },
-        },
-        'axes': axes,
-        'original_metadata': original_metadata,
-        'mapping': mapping}
-    return dictionary
+    return dc
 
 
 def _guess_units_from_mode(objects_dict, header):
@@ -683,7 +683,7 @@ def _get_degree(value):
 
 
 def _get_date_time(value):
-    dt = datetime.strptime(value, "%a %b %d %H:%M:%S %Y")
+    dt = parser.parse(value)
     return dt.date().isoformat(), dt.time().isoformat()
 
 
@@ -709,5 +709,7 @@ mapping = {
     "ObjectInfo.ExperimentalConditions.MicroscopeConditions.Tilt1": (
         "Acquisition_instrument.TEM.tilt_stage",
         _get_degree),
-    "ObjectInfo.ExperimentalDescription.User": ("General.authors", None),
+    "ObjectInfo.ExperimentalDescription.User": (
+        "General.authors",
+        None),
 }
