@@ -1,5 +1,6 @@
 from operator import attrgetter
 import numpy as np
+from dask.array import Array as dArray
 from hyperspy.misc.utils import attrsetter
 from hyperspy.misc.export_dictionary import parse_flag_string
 
@@ -207,8 +208,15 @@ class FancySlicing(object):
 
     def _slicer(self, slices, isNavigation=None, out=None):
         array_slices = self._get_array_slices(slices, isNavigation)
+        new_data = self.data[array_slices]
+        if new_data.size == 1 and new_data.dtype is np.dtype('O'):
+            if isinstance(new_data[0], (np.ndarray, dArray)):
+                return self.__class__(new_data[0]).transpose(navigation_axes=0)
+            else:
+                return new_data[0]
+
         if out is None:
-            _obj = self._deepcopy_with_new_data(self.data[array_slices],
+            _obj = self._deepcopy_with_new_data(new_data,
                                                 copy_variance=True)
             _to_remove = []
             for slice_, axis in zip(array_slices, _obj.axes_manager._axes):
@@ -220,7 +228,7 @@ class FancySlicing(object):
             for _ind in reversed(sorted(_to_remove)):
                 _obj._remove_axis(_ind)
         else:
-            out.data = self.data[array_slices]
+            out.data = new_data
             _obj = out
             i = 0
             for slice_, axis_src in zip(array_slices, self.axes_manager._axes):
