@@ -17,6 +17,7 @@
 # along with  HyperSpy.  If not, see <http://www.gnu.org/licenses/>.
 
 from operator import attrgetter
+import warnings
 import inspect
 import copy
 import types
@@ -26,6 +27,8 @@ import collections
 import tempfile
 import unicodedata
 from contextlib import contextmanager
+from ..misc.signal_tools import broadcast_signals
+from ..exceptions import VisibleDeprecationWarning
 
 import numpy as np
 
@@ -385,6 +388,8 @@ class DictionaryTreeBrowser(object):
                 elif isinstance(item_['_dtb_value_'], BaseSignal):
                     item = item_['_dtb_value_']._to_dictionary()
                     key = '_sig_' + key
+                elif hasattr(item_['_dtb_value_'], '_to_dictionary'):
+                    item = item_['_dtb_value_']._to_dictionary()
                 else:
                     item = item_['_dtb_value_']
                 par_dict.__setitem__(key, item)
@@ -754,7 +759,7 @@ def without_nans(data):
 
 
 def stack(signal_list, axis=None, new_axis_name='stack_element',
-          mmap=False, mmap_dir=None,):
+          lazy=None, **kwargs):
     """Concatenate the signals in the list over a given axis or a new axis.
 
     The title is set to that of the first signal in the list.
@@ -773,17 +778,9 @@ def stack(signal_list, axis=None, new_axis_name='stack_element',
         If an axis with this name already
         exists it automatically append '-i', where `i` are integers,
         until it finds a name that is not yet in use.
-    mmap: bool
-        If True and stack is True, then the data is stored
-        in a memory-mapped temporary file.The memory-mapped data is
-        stored on disk, and not directly loaded into memory.
-        Memory mapping is especially useful for accessing small
-        fragments of large files without reading the entire file into
-        memory.
-    mmap_dir : string
-        If mmap_dir is not None, and stack and mmap are True, the memory
-        mapped file will be created in the given directory,
-        otherwise the default directory is used.
+    lazy: {bool, None}
+        Returns a LazySignal if True. If None, only returns lazy rezult if at
+        least one is lazy.
 
     Returns
     -------
@@ -802,95 +799,115 @@ def stack(signal_list, axis=None, new_axis_name='stack_element',
            [10, 11, 12, 13, 14, 15, 16, 17, 18, 19]])
 
     """
+    from itertools import zip_longest
+    from hyperspy.signals import BaseSignal
+    import dask.array as da
+    from numbers import Number
+    # TODO: remove next time
+    deprecated = ['mmap', 'mmap_dir']
+    warn_str = "'{}' argument is deprecated, please use 'lazy' instead"
+    for k in deprecated:
+        if k in kwargs:
+            lazy = True
+            warnings.warn(warn_str.format(k), VisibleDeprecationWarning)
 
     axis_input = copy.deepcopy(axis)
+    signal_list = list(signal_list)
+    # Get the real signal with the most axes to get metadata/class/etc
+    # first = sorted(filter(lambda _s: isinstance(_s, BaseSignal), signal_list),
+    #                key=lambda _s: _s.data.ndim)[-1]
+    first = next(filter(lambda _s: isinstance(_s, BaseSignal), signal_list))
 
-    for i, obj in enumerate(signal_list):
-        if i == 0:
-            if axis is None:
-                original_shape = obj.data.shape
-                stack_shape = tuple([len(signal_list), ]) + original_shape
-                tempf = None
-                if mmap is False:
-                    data = np.empty(stack_shape,
-                                    dtype=obj.data.dtype)
-                else:
-                    tempf = tempfile.NamedTemporaryFile(
-                        dir=mmap_dir)
-                    data = np.memmap(tempf,
-                                     dtype=obj.data.dtype,
-                                     mode='w+',
-                                     shape=stack_shape,)
+    # Cast numbers as signals. Will broadcast later
 
-                signal = type(obj)(data=data)
-                signal.axes_manager._axes[
-                    1:] = copy.deepcopy(
-                    obj.axes_manager._axes)
-                axis_name = new_axis_name
-                axis_names = [axis_.name for axis_ in
-                              signal.axes_manager._axes[1:]]
-                j = 1
-                while axis_name in axis_names:
-                    axis_name = new_axis_name + "_%i" % j
-                    j += 1
-                eaxis = signal.axes_manager._axes[0]
-                eaxis.name = axis_name
-                eaxis.navigate = True  # This triggers _update_parameters
-                signal.metadata = copy.deepcopy(obj.metadata)
-                # Get the title from 1st object
-                signal.metadata.General.title = (
-                    "Stack of " + obj.metadata.General.title)
-                signal.original_metadata = DictionaryTreeBrowser({})
-            else:
-                axis = obj.axes_manager[axis]
-                signal = obj.deepcopy()
+    for i, _s in enumerate(signal_list):
+        if isinstance(_s, BaseSignal):
+            pass
+        elif isinstance(_s, Number):
+            sig = BaseSignal(_s)
+            signal_list[i] = sig
+        else:
+            raise ValueError("{} type cannot be stacked.".format(type(_s)))
 
-            signal.original_metadata.add_node('stack_elements')
+    if lazy is None:
+        lazy = any(_s._lazy for _s in signal_list)
+    if not isinstance(lazy, bool):
+        raise ValueError("'lazy' argument has to be None, True or False")
 
-        # Store parameters
-        signal.original_metadata.stack_elements.add_node(
-            'element%i' % i)
-        node = signal.original_metadata.stack_elements[
-            'element%i' % i]
-        node.original_metadata = \
-            obj.original_metadata.as_dictionary()
-        node.metadata = \
-            obj.metadata.as_dictionary()
-
-        if axis is None:
-            if obj.data.shape != original_shape:
-                raise IOError(
-                    "Only files with data of the same shape can be stacked")
-            signal.data[i, ...] = obj.data
-            del obj
-    if axis is not None:
-        signal.data = np.concatenate([signal_.data for signal_ in signal_list],
-                                     axis=axis.index_in_array)
+    # Cast all as lazy if required
+    for i, _s in enumerate(signal_list):
+        if not _s._lazy:
+            signal_list[i] = _s.as_lazy()
+    if len(signal_list) > 1:
+        newlist = broadcast_signals(*signal_list, ignore_axis=axis_input)
+        if axis is not None:
+            step_sizes = [s.axes_manager[axis].size for s in newlist]
+            axis = newlist[0].axes_manager[axis]
+        datalist = [s.data for s in newlist]
+        newdata = da.stack(datalist, axis=0) if axis is None else \
+            da.concatenate(datalist, axis=axis.index_in_array)
+        if axis_input is None:
+            signal = first.__class__(newdata)
+            signal._lazy = True
+            signal._assign_subclass()
+            signal.axes_manager._axes[1:] = copy.deepcopy(
+                newlist[0].axes_manager._axes)
+            axis_name = new_axis_name
+            axis_names = [
+                axis_.name for axis_ in signal.axes_manager._axes[
+                    1:]]
+            j = 1
+            while axis_name in axis_names:
+                axis_name = new_axis_name + "_%i" % j
+                j += 1
+            eaxis = signal.axes_manager._axes[0]
+            eaxis.name = axis_name
+            eaxis.navigate = True  # This triggers _update_parameters
+            signal.metadata = copy.deepcopy(first.metadata)
+            # Get the title from 1st object
+            signal.metadata.General.title = (
+                "Stack of " + first.metadata.General.title)
+            signal.original_metadata = DictionaryTreeBrowser({})
+        else:
+            signal = newlist[0]._deepcopy_with_new_data(newdata)
+            signal._lazy = True
+            signal._assign_subclass()
         signal.get_dimensions_from_data()
+        signal.original_metadata.add_node('stack_elements')
 
-    if axis_input is None:
-        axis_input = signal.axes_manager[-1 + 1j].index_in_axes_manager
-        step_sizes = 1
-    else:
-        step_sizes = [obj.axes_manager[axis_input].size
-                      for obj in signal_list]
+        for i, obj in enumerate(signal_list):
+            signal.original_metadata.stack_elements.add_node('element%i' % i)
+            node = signal.original_metadata.stack_elements['element%i' % i]
+            node.original_metadata = \
+                obj.original_metadata.as_dictionary()
+            node.metadata = \
+                obj.metadata.as_dictionary()
 
-    signal.metadata._HyperSpy.set_item(
-        'Stacking_history.axis',
-        axis_input)
-    signal.metadata._HyperSpy.set_item(
-        'Stacking_history.step_sizes',
-        step_sizes)
-    from hyperspy.signal import BaseSignal
-    if np.all([s.metadata.has_item('Signal.Noise_properties.variance')
-               for s in signal_list]):
-        if np.all([isinstance(s.metadata.Signal.Noise_properties.variance, BaseSignal)
-                   for s in signal_list]):
-            variance = stack(
-                [s.metadata.Signal.Noise_properties.variance for s in signal_list], axis)
+        if axis_input is None:
+            axis_input = signal.axes_manager[-1 + 1j].index_in_axes_manager
+            step_sizes = 1
+
+        signal.metadata._HyperSpy.set_item('Stacking_history.axis', axis_input)
+        signal.metadata._HyperSpy.set_item('Stacking_history.step_sizes',
+                                           step_sizes)
+        if np.all([
+                s.metadata.has_item('Signal.Noise_properties.variance')
+                for s in signal_list
+        ]):
+            variance = stack([
+                s.metadata.Signal.Noise_properties.variance for s in signal_list
+            ], axis)
             signal.metadata.set_item(
-                'Signal.Noise_properties.variance',
-                variance)
+                'Signal.Noise_properties.variance', variance)
+    else:
+        signal = signal_list[0]
+
+    # Leave as lazy or compute
+    if lazy:
+        signal = signal.as_lazy()
+    else:
+        signal.compute(False)
+
     return signal
 
 
@@ -927,3 +944,89 @@ def transpose(*args, signal_axes=None, navigation_axes=None, optimize=False):
     return [sig.transpose(signal_axes=signal_axes,
                           navigation_axes=navigation_axes,
                           optimize=optimize) for sig in args]
+
+
+def create_map_objects(function, nav_size, iterating_kwargs, **kwargs):
+    """To be used in _map_iterate of BaseSignal and LazySignal.
+
+    Moved to a separate method to reduce code duplication.
+    """
+    from hyperspy.signal import BaseSignal
+    from itertools import repeat
+
+    iterators = tuple(signal[1]._iterate_signal()
+                      if isinstance(signal[1], BaseSignal) else signal[1]
+                      for signal in iterating_kwargs)
+    # make all kwargs iterating for simplicity:
+    iterating = tuple(key for key, value in iterating_kwargs)
+    for k, v in kwargs.items():
+        if k not in iterating:
+            iterating += k,
+            iterators += repeat(v, nav_size),
+
+    def figure_out_kwargs(data):
+        _kwargs = {k: v for k, v in zip(iterating, data[1:])}
+        for k, v in iterating_kwargs:
+            if (isinstance(v, BaseSignal) and
+                isinstance(_kwargs[k], np.ndarray) and
+                    len(_kwargs[k]) == 1):
+                _kwargs[k] = _kwargs[k][0]
+        return data[0], _kwargs
+
+    def func(*args):
+        dat, these_kwargs = figure_out_kwargs(*args)
+        return function(dat, **these_kwargs)
+
+    return func, iterators
+
+
+def map_result_construction(signal,
+                            inplace,
+                            result,
+                            ragged,
+                            sig_shape=None,
+                            lazy=False):
+    from hyperspy.signals import (BaseSignal, LazySignal)
+    res = None
+    if inplace:
+        sig = signal
+    else:
+        res = sig = signal._deepcopy_with_new_data()
+    if ragged:
+        sig.data = result
+        sig.axes_manager.remove(sig.axes_manager.signal_axes)
+        sig.__class__ = LazySignal if lazy else BaseSignal
+        sig.__init__(**sig._to_dictionary(add_models=True))
+
+    else:
+        if not sig._lazy and sig.data.shape == result.shape and np.can_cast(
+                result.dtype, sig.data.dtype):
+            sig.data[:] = result
+        else:
+            sig.data = result
+
+        # remove if too many axes
+        sig.axes_manager.remove(sig.axes_manager.signal_axes[len(sig_shape):])
+        # add additional required axes
+        for ind in range(
+                len(sig_shape) - sig.axes_manager.signal_dimension, 0, -1):
+            sig.axes_manager._append_axis(sig_shape[-ind], navigate=False)
+    sig.get_dimensions_from_data()
+    return res
+
+
+def multiply(iterable):
+    """Return product of sequence of numbers.
+
+    Equivalent of functools.reduce(operator.mul, iterable, 1).
+
+    >>> product([2**8, 2**30])
+    274877906944
+    >>> product([])
+    1
+
+    """
+    prod = 1
+    for i in iterable:
+        prod *= i
+    return prod
