@@ -42,17 +42,10 @@ def _get_weight(element, line, weight_line=None):
     if weight_line is None:
         weight_line = elements_db[
             element]['Atomic_properties']['Xray_lines'][line]['weight']
-    return lambda x: x * weight_line
+    return "x * {}".format(weight_line)
 
 
-def _get_iweight(element, line, weight_line=None):
-    if weight_line is None:
-        weight_line = elements_db[
-            element]['Atomic_properties']['Xray_lines'][line]['weight']
-    return lambda x: x / weight_line
-
-
-def _get_sigma(E, E_ref, units_factor):
+def _get_sigma(E, E_ref, units_factor, return_f=False):
     """
     Calculates an approximate sigma value, accounting for peak broadening due
     to the detector, for a peak at energy E given a known width at a reference
@@ -84,17 +77,21 @@ def _get_sigma(E, E_ref, units_factor):
         Microanalysis", Plenum, third edition, p 315.
     """
     energy2sigma_factor = 2.5 / (eV2keV * (sigma2fwhm**2))
-    return lambda sig_ref: math.sqrt(abs(
-        energy2sigma_factor * (E - E_ref) * units_factor +
-        np.power(sig_ref, 2)))
+    if return_f:
+        return lambda sig_ref: math.sqrt(abs(
+            energy2sigma_factor * (E - E_ref) * units_factor +
+            np.power(sig_ref, 2)))
+    else:
+        return "sqrt(abs({} * ({} - {}) * {} + sig_ref ** 2))".format(
+            energy2sigma_factor, E, E_ref, units_factor)
 
 
 def _get_offset(diff):
-    return lambda E: E + diff
+    return "x + {}".format(diff)
 
 
 def _get_scale(E1, E_ref1, fact):
-    return lambda E: E1 + fact * (E - E_ref1)
+    return "{} + {} * (x - {})".format(E1, fact, E_ref1)
 
 
 class EDSModel(Model1D):
@@ -264,9 +261,7 @@ class EDSModel(Model1D):
                         component_sub.centre.free = False
                         component_sub.sigma.free = False
                         component_sub.name = xray_sub
-                        component_sub.A.twin_function = _get_weight(
-                            element, li)
-                        component_sub.A.twin_inverse_function = _get_iweight(
+                        component_sub.A.twin_function_expr = _get_weight(
                             element, li)
                         component_sub.A.twin = component.A
                         self.append(component_sub)
@@ -412,11 +407,10 @@ class EDSModel(Model1D):
             else:
                 component.sigma.free = True
                 E = component.centre.value
-                component.sigma.twin_function = _get_sigma(
-                    E, E_ref, self.units_factor)
-                component.sigma.twin_inverse_function = _get_sigma(
+                component.sigma.twin_inverse_function_expr = _get_sigma(
                     E_ref, E, self.units_factor)
-                component.sigma.twin = component_ref.sigma
+                component.sigma.twin_function_expr = _get_sigma(
+                    E, E_ref, self.units_factor)
 
     def _set_energy_resolution(self, xray_lines, *args, **kwargs):
         """
@@ -434,7 +428,8 @@ class EDSModel(Model1D):
                                                                    'auto')
         FWHM_MnKa_old *= eV2keV / self.units_factor
         get_sigma_Mn_Ka = _get_sigma(
-            energy_Mn_Ka, self[xray_lines[0]].centre.value, self.units_factor)
+            energy_Mn_Ka, self[xray_lines[0]].centre.value, self.units_factor,
+            return_f=True)
         FWHM_MnKa = get_sigma_Mn_Ka(self[xray_lines[0]].sigma.value
                                     ) * eV2keV / self.units_factor * sigma2fwhm
         if FWHM_MnKa < 110:
@@ -443,8 +438,9 @@ class EDSModel(Model1D):
         else:
             self.signal.set_microscope_parameters(
                 energy_resolution_MnKa=FWHM_MnKa)
-            warnings.warn("Energy resolution (FWHM at Mn Ka) changed from " +
-                          "%lf to %lf eV" % (FWHM_MnKa_old, FWHM_MnKa))
+            _logger.info("Energy resolution (FWHM at Mn Ka) changed from " +
+                         "{:.2f} to {:.2f} eV".format(
+                             FWHM_MnKa_old, FWHM_MnKa))
             for component in self:
                 if component.isbackground is False:
                     line_FWHM = self.signal._get_line_energy(
@@ -475,9 +471,8 @@ class EDSModel(Model1D):
                 component.centre.free = True
                 E = component.centre.value
                 fact = float(ax.value2index(E)) / ax.value2index(E_ref)
-                component.centre.twin_function = _get_scale(E, E_ref, fact)
-                component.centre.twin_inverse_function = _get_scale(
-                    E_ref, E, 1. / fact)
+                component.centre.twin_function_expr = _get_scale(
+                    E, E_ref, fact)
                 component.centre.twin = component_ref.centre
                 ref.append(E)
         return ref
@@ -531,8 +526,7 @@ class EDSModel(Model1D):
                 component.centre.free = True
                 E = component.centre.value
                 diff = E_ref - E
-                component.centre.twin_function = _get_offset(-diff)
-                component.centre.twin_inverse_function = _get_offset(diff)
+                component.centre.twin_function_expr = _get_offset(-diff)
                 component.centre.twin = component_ref.centre
                 ref.append(E)
         return ref
@@ -657,9 +651,7 @@ class EDSModel(Model1D):
                         component_sub.A.bmin = 1e-10
                         component_sub.A.bmax = None
                         weight_line = component_sub.A.value / component.A.value
-                        component_sub.A.twin_function = _get_weight(
-                            element, li, weight_line)
-                        component_sub.A.twin_inverse_function = _get_iweight(
+                        component_sub.A.twin_function_expr = _get_weight(
                             element, li, weight_line)
                         component_sub.A.twin = component.A
                     else:
@@ -862,10 +854,6 @@ class EDSModel(Model1D):
                  line_energy,
                  self.signal.axes_manager.signal_axes[0].units,
                  self.signal.metadata.General.title))
-            if img.axes_manager.navigation_dimension >= 2:
-                img = img.as_signal2D([0, 1])
-            elif img.axes_manager.navigation_dimension == 1:
-                img.axes_manager.set_signal_dimension(1)
             if plot_result and img.axes_manager.signal_dimension == 0:
                 print("%s at %s %s : Intensity = %.2f"
                       % (xray_line,
