@@ -1,10 +1,11 @@
 import os
 
-import numpy as np
 import json
-from nose.plugins.skip import SkipTest
 
-import nose.tools as nt
+import numpy as np
+import pytest
+from numpy.testing import assert_allclose
+
 from hyperspy.io import load
 from hyperspy import signals
 from hyperspy.misc.test_utils import assert_deep_almost_equal
@@ -18,16 +19,9 @@ np_file = ['P45_16bit.npy', 'P45_16bit_ds.npy']
 
 my_path = os.path.dirname(__file__)
 
-try:
-    import lxml
-    skip_test = False
-except ImportError:
-    skip_test = True
-
 
 def test_load_16bit():
-    if skip_test:
-        raise SkipTest
+    lxml = pytest.importorskip("lxml")
     # test bcf from hyperspy load function level
     # some of functions can be not covered
     # it cant use cython parsing implementation, as it is not compiled
@@ -36,68 +30,65 @@ def test_load_16bit():
     s = load(filename)
     bse, sei, hype = s
     # Bruker saves all images in true 16bit:
-    nt.assert_true(bse.data.dtype == np.uint16)
-    nt.assert_true(sei.data.dtype == np.uint16)
-    nt.assert_true(bse.data.shape == (75, 100))
+    assert bse.data.dtype == np.uint16
+    assert sei.data.dtype == np.uint16
+    assert bse.data.shape == (75, 100)
     np_filename = os.path.join(my_path, 'bcf_data', np_file[0])
     np.testing.assert_array_equal(hype.data[:22, :22, 222],
                                   np.load(np_filename))
-    nt.assert_true(hype.data.shape == (75, 100, 2048))
+    assert hype.data.shape == (75, 100, 2048)
 
 
 def test_load_16bit_reduced():
-    if skip_test:
-        raise SkipTest
+    lxml = pytest.importorskip("lxml")
     filename = os.path.join(my_path, 'bcf_data', test_files[0])
     print('testing downsampled 16bit bcf...')
     s = load(filename, downsample=4, cutoff_at_kV=10)
     bse, sei, hype = s
     # sem images never are downsampled
-    nt.assert_true(bse.data.shape == (75, 100))
+    assert bse.data.shape == (75, 100)
     np_filename = os.path.join(my_path, 'bcf_data', np_file[1])
     np.testing.assert_array_equal(hype.data[:2, :2, 222],
                                   np.load(np_filename))
-    nt.assert_true(hype.data.shape == (19, 25, 1047))
+    assert hype.data.shape == (19, 25, 1047)
     # Bruker saves all images in true 16bit:
-    nt.assert_true(bse.data.dtype == np.uint16)
-    nt.assert_true(sei.data.dtype == np.uint16)
+    assert bse.data.dtype == np.uint16
+    assert sei.data.dtype == np.uint16
     # hypermaps should always return unsigned integers:
-    nt.assert_true(str(hype.data.dtype)[0] == 'u')
+    assert str(hype.data.dtype)[0] == 'u'
 
 
 def test_load_8bit():
-    if skip_test:
-        raise SkipTest
+    lxml = pytest.importorskip("lxml")
     for bcffile in test_files[1:3]:
         filename = os.path.join(my_path, 'bcf_data', bcffile)
         print('testing simple 8bit bcf...')
         s = load(filename)
         bse, sei, hype = s
         # Bruker saves all images in true 16bit:
-        nt.assert_true(bse.data.dtype == np.uint16)
-        nt.assert_true(sei.data.dtype == np.uint16)
+        assert bse.data.dtype == np.uint16
+        assert sei.data.dtype == np.uint16
         # hypermaps should always return unsigned integers:
-        nt.assert_true(str(hype.data.dtype)[0] == 'u')
+        assert str(hype.data.dtype)[0] == 'u'
 
 
 def test_hyperspy_wrap():
-    if skip_test:
-        raise SkipTest
+    lxml = pytest.importorskip("lxml")
     filename = os.path.join(my_path, 'bcf_data', test_files[0])
     print('testing bcf wrap to hyperspy signal...')
     hype = load(filename, select_type='spectrum')
-    nt.assert_almost_equal(
+    assert_allclose(
         hype.axes_manager[0].scale,
         8.7367850619778,
-        places=12)
-    nt.assert_almost_equal(
+        atol=1E-12)
+    assert_allclose(
         hype.axes_manager[1].scale,
         8.7367850619778,
-        places=12)
-    nt.assert_equal(hype.axes_manager[1].units, 'µm')
-    nt.assert_almost_equal(hype.axes_manager[2].scale, 0.010001)
-    nt.assert_almost_equal(hype.axes_manager[2].offset, -0.472397235)
-    nt.assert_equal(hype.axes_manager[2].units, 'keV')
+        atol=1E-12)
+    assert hype.axes_manager[1].units == 'µm'
+    assert_allclose(hype.axes_manager[2].scale, 0.010001)
+    assert_allclose(hype.axes_manager[2].offset, -0.472397235)
+    assert hype.axes_manager[2].units == 'keV'
 
     md_ref = {'_HyperSpy': {'Folding': {'original_shape': None,
                                         'unfolded': False,
@@ -127,35 +118,33 @@ def test_hyperspy_wrap():
                                 'bcf_data',
                                 'test_original_metadata.json')
     with open(filename_omd) as fn:
-        #original_metadata:
+        # original_metadata:
         omd_ref = json.load(fn)
     assert_deep_almost_equal(hype.metadata.as_dictionary(), md_ref)
     assert_deep_almost_equal(hype.original_metadata.as_dictionary(), omd_ref)
-    nt.assert_equal(hype.metadata.General.date, "2016-04-01")
-    nt.assert_equal(hype.metadata.General.time, "17:05:03")
-    nt.assert_equal(hype.metadata.Signal.quantity, "X-rays (Counts)")
+    assert hype.metadata.General.date == "2016-04-01"
+    assert hype.metadata.General.time == "17:05:03"
+    assert hype.metadata.Signal.quantity == "X-rays (Counts)"
 
 
 def test_hyperspy_wrap_downsampled():
-    if skip_test:
-        raise SkipTest
+    lxml = pytest.importorskip("lxml")
     filename = os.path.join(my_path, 'bcf_data', test_files[0])
     print('testing bcf wrap to hyperspy signal...')
     hype = load(filename, select_type='spectrum', downsample=5)
-    nt.assert_almost_equal(
+    assert_allclose(
         hype.axes_manager[0].scale,
         43.683925309889,
-        places=12)
-    nt.assert_almost_equal(
+        atol=1E-12)
+    assert_allclose(
         hype.axes_manager[1].scale,
         43.683925309889,
-        places=12)
-    nt.assert_true(hype.axes_manager[1].units == 'µm')
+        atol=1E-12)
+    assert hype.axes_manager[1].units == 'µm'
 
 
 def test_fast_bcf():
-    if skip_test:
-        raise SkipTest
+    lxml = pytest.importorskip("lxml")
     from hyperspy.io_plugins import bcf
 
     for bcffile in test_files:
@@ -171,32 +160,30 @@ def test_fast_bcf():
 
 
 def test_get_mode():
-    if skip_test:
-        raise SkipTest
+    lxml = pytest.importorskip("lxml")
     filename = os.path.join(my_path, 'bcf_data', test_files[0])
     s = load(filename, select_type='spectrum', instrument='SEM')
-    nt.assert_equal(s.metadata.Signal.signal_type, "EDS_SEM")
-    nt.assert_true(isinstance(s, signals.EDSSEMSpectrum))
+    assert s.metadata.Signal.signal_type == "EDS_SEM"
+    assert isinstance(s, signals.EDSSEMSpectrum)
 
     filename = os.path.join(my_path, 'bcf_data', test_files[0])
     s = load(filename, select_type='spectrum', instrument='TEM')
-    nt.assert_equal(s.metadata.Signal.signal_type, "EDS_TEM")
-    nt.assert_true(isinstance(s, signals.EDSTEMSpectrum))
+    assert s.metadata.Signal.signal_type == "EDS_TEM"
+    assert isinstance(s, signals.EDSTEMSpectrum)
 
     filename = os.path.join(my_path, 'bcf_data', test_files[0])
     s = load(filename, select_type='spectrum')
-    nt.assert_equal(s.metadata.Signal.signal_type, "EDS_SEM")
-    nt.assert_true(isinstance(s, signals.EDSSEMSpectrum))
+    assert s.metadata.Signal.signal_type == "EDS_SEM"
+    assert isinstance(s, signals.EDSSEMSpectrum)
 
     filename = os.path.join(my_path, 'bcf_data', test_files[3])
     s = load(filename, select_type='spectrum')
-    nt.assert_equal(s.metadata.Signal.signal_type, "EDS_TEM")
-    nt.assert_true(isinstance(s, signals.EDSTEMSpectrum))
-    
+    assert s.metadata.Signal.signal_type == "EDS_TEM"
+    assert isinstance(s, signals.EDSTEMSpectrum)
+
 
 def test_wrong_file():
-    if skip_test:
-        raise SkipTest
+    lxml = pytest.importorskip("lxml")
     filename = os.path.join(my_path, 'bcf_data', 'Nope.bcf')
-    with nt.assert_raises(TypeError):
+    with pytest.raises(TypeError):
         load(filename)
