@@ -2,8 +2,9 @@ import os
 import tempfile
 
 import numpy as np
-import nose.tools as nt
+
 import traits.api as t
+from numpy.testing import assert_allclose
 
 import hyperspy.api as hs
 from hyperspy.misc.test_utils import assert_deep_almost_equal
@@ -26,36 +27,36 @@ def _test_rgba16(import_local_tifffile=False):
     s = hs.load(os.path.join(MY_PATH2, "test_rgba16.tif"),
                 import_local_tifffile=import_local_tifffile)
     data = np.load(os.path.join(MY_PATH, "npy_files", "test_rgba16.npy"))
-    nt.assert_true((s.data == data).all())
-    nt.assert_equal(s.axes_manager[0].units, t.Undefined)
-    nt.assert_equal(s.axes_manager[1].units, t.Undefined)
-    nt.assert_equal(s.axes_manager[2].units, t.Undefined)
-    nt.assert_almost_equal(s.axes_manager[0].scale, 1.0, places=5)
-    nt.assert_almost_equal(s.axes_manager[1].scale, 1.0, places=5)
-    nt.assert_almost_equal(s.axes_manager[2].scale, 1.0, places=5)
+    assert (s.data == data).all()
+    assert s.axes_manager[0].units == t.Undefined
+    assert s.axes_manager[1].units == t.Undefined
+    assert s.axes_manager[2].units == t.Undefined
+    assert_allclose(s.axes_manager[0].scale, 1.0, atol=1E-5)
+    assert_allclose(s.axes_manager[1].scale, 1.0, atol=1E-5)
+    assert_allclose(s.axes_manager[2].scale, 1.0, atol=1E-5)
 
 
 def _compare_signal_shape_data(s0, s1):
-    nt.assert_equal(s0.data.shape, s1.data.shape)
+    assert s0.data.shape == s1.data.shape
     np.testing.assert_equal(s0.data, s1.data)
 
 
 def test_read_unit_um():
     # Load DM file and save it as tif
     s = hs.load(os.path.join(MY_PATH2, 'test_dm_image_um_unit.dm3'))
-    nt.assert_equal(s.axes_manager[0].units, 'µm')
-    nt.assert_equal(s.axes_manager[1].units, 'µm')
-    nt.assert_almost_equal(s.axes_manager[0].scale, 0.16867, places=5)
-    nt.assert_almost_equal(s.axes_manager[1].scale, 0.16867, places=5)
+    assert s.axes_manager[0].units == 'µm'
+    assert s.axes_manager[1].units == 'µm'
+    assert_allclose(s.axes_manager[0].scale, 0.16867, atol=1E-5)
+    assert_allclose(s.axes_manager[1].scale, 0.16867, atol=1E-5)
     with tempfile.TemporaryDirectory() as tmpdir:
         fname = os.path.join(tmpdir, 'tiff_files', 'test_export_um_unit.tif')
         s.save(fname, overwrite=True, export_scale=True)
         # load tif file
         s2 = hs.load(fname, import_local_tifffile=True)
-        nt.assert_equal(s.axes_manager[0].units, 'µm')
-        nt.assert_equal(s.axes_manager[1].units, 'µm')
-        nt.assert_almost_equal(s2.axes_manager[0].scale, 0.16867, places=5)
-        nt.assert_almost_equal(s2.axes_manager[1].scale, 0.16867, places=5)
+        assert s.axes_manager[0].units == 'µm'
+        assert s.axes_manager[1].units == 'µm'
+        assert_allclose(s2.axes_manager[0].scale, 0.16867, atol=1E-5)
+        assert_allclose(s2.axes_manager[1].scale, 0.16867, atol=1E-5)
 
 
 def test_write_read_intensity_axes_DM():
@@ -86,80 +87,80 @@ def _test_read_unit_from_imagej(import_local_tifffile=False):
     fname = os.path.join(MY_PATH, 'tiff_files',
                          'test_loading_image_saved_with_imageJ.tif')
     s = hs.load(fname, import_local_tifffile=import_local_tifffile)
-    nt.assert_equal(s.axes_manager[0].units, 'micron')
-    nt.assert_equal(s.axes_manager[1].units, 'micron')
-    nt.assert_almost_equal(s.axes_manager[0].scale, 0.16867, places=5)
-    nt.assert_almost_equal(s.axes_manager[1].scale, 0.16867, places=5)
+    assert s.axes_manager[0].units == 'micron'
+    assert s.axes_manager[1].units == 'micron'
+    assert_allclose(s.axes_manager[0].scale, 0.16867, atol=1E-5)
+    assert_allclose(s.axes_manager[1].scale, 0.16867, atol=1E-5)
 
 
 def test_read_unit_from_imagej_stack(import_local_tifffile=False):
     fname = os.path.join(MY_PATH, 'tiff_files',
                          'test_loading_image_saved_with_imageJ_stack.tif')
     s = hs.load(fname, import_local_tifffile=import_local_tifffile)
-    nt.assert_equal(s.data.shape, (2, 68, 68))
-    nt.assert_equal(s.axes_manager[0].units, t.Undefined)
-    nt.assert_equal(s.axes_manager[1].units, 'micron')
-    nt.assert_equal(s.axes_manager[2].units, 'micron')
-    nt.assert_almost_equal(s.axes_manager[0].scale, 2.5, places=5)
-    nt.assert_almost_equal(s.axes_manager[1].scale, 0.16867, places=5)
-    nt.assert_almost_equal(s.axes_manager[2].scale, 0.16867, places=5)
+    assert s.data.shape == (2, 68, 68)
+    assert s.axes_manager[0].units == t.Undefined
+    assert s.axes_manager[1].units == 'micron'
+    assert s.axes_manager[2].units == 'micron'
+    assert_allclose(s.axes_manager[0].scale, 2.5, atol=1E-5)
+    assert_allclose(s.axes_manager[1].scale, 0.16867, atol=1E-5)
+    assert_allclose(s.axes_manager[2].scale, 0.16867, atol=1E-5)
 
 
 def test_read_unit_from_DM_stack(import_local_tifffile=False):
     fname = os.path.join(MY_PATH, 'tiff_files',
                          'test_loading_image_saved_with_DM_stack.tif')
     s = hs.load(fname, import_local_tifffile=import_local_tifffile)
-    nt.assert_equal(s.data.shape, (2, 68, 68))
-    nt.assert_equal(s.axes_manager[0].units, 's')
-    nt.assert_equal(s.axes_manager[1].units, 'µm')
-    nt.assert_equal(s.axes_manager[2].units, 'µm')
-    nt.assert_almost_equal(s.axes_manager[0].scale, 2.5, places=5)
-    nt.assert_almost_equal(s.axes_manager[1].scale, 0.16867, places=5)
-    nt.assert_almost_equal(s.axes_manager[2].scale, 1.68674, places=5)
+    assert s.data.shape == (2, 68, 68)
+    assert s.axes_manager[0].units == 's'
+    assert s.axes_manager[1].units == 'µm'
+    assert s.axes_manager[2].units == 'µm'
+    assert_allclose(s.axes_manager[0].scale, 2.5, atol=1E-5)
+    assert_allclose(s.axes_manager[1].scale, 0.16867, atol=1E-5)
+    assert_allclose(s.axes_manager[2].scale, 1.68674, atol=1E-5)
     with tempfile.TemporaryDirectory() as tmpdir:
         fname2 = os.path.join(
             tmpdir, 'test_loading_image_saved_with_DM_stack2.tif')
         s.save(fname2, overwrite=True)
         s2 = hs.load(fname2)
         _compare_signal_shape_data(s, s2)
-        nt.assert_equal(s2.axes_manager[0].units, s.axes_manager[0].units)
-        nt.assert_equal(s2.axes_manager[1].units, 'micron')
-        nt.assert_equal(s2.axes_manager[2].units, 'micron')
-        nt.assert_almost_equal(
-            s2.axes_manager[0].scale, s.axes_manager[0].scale, places=5)
-        nt.assert_almost_equal(
-            s2.axes_manager[1].scale, s.axes_manager[1].scale, places=5)
-        nt.assert_almost_equal(
-            s2.axes_manager[2].scale, s.axes_manager[2].scale, places=5)
-        nt.assert_almost_equal(
-            s2.axes_manager[0].offset, s.axes_manager[0].offset, places=5)
-        nt.assert_almost_equal(
-            s2.axes_manager[1].offset, s.axes_manager[1].offset, places=5)
-        nt.assert_almost_equal(
-            s2.axes_manager[2].offset, s.axes_manager[2].offset, places=5)
+        assert s2.axes_manager[0].units == s.axes_manager[0].units
+        assert s2.axes_manager[1].units == 'micron'
+        assert s2.axes_manager[2].units == 'micron'
+        assert_allclose(
+            s2.axes_manager[0].scale, s.axes_manager[0].scale, atol=1E-5)
+        assert_allclose(
+            s2.axes_manager[1].scale, s.axes_manager[1].scale, atol=1E-5)
+        assert_allclose(
+            s2.axes_manager[2].scale, s.axes_manager[2].scale, atol=1E-5)
+        assert_allclose(
+            s2.axes_manager[0].offset, s.axes_manager[0].offset, atol=1E-5)
+        assert_allclose(
+            s2.axes_manager[1].offset, s.axes_manager[1].offset, atol=1E-5)
+        assert_allclose(
+            s2.axes_manager[2].offset, s.axes_manager[2].offset, atol=1E-5)
 
 
 def test_read_unit_from_imagej_stack_no_scale(import_local_tifffile=False):
     fname = os.path.join(MY_PATH, 'tiff_files',
                          'test_loading_image_saved_with_imageJ_stack_no_scale.tif')
     s = hs.load(fname, import_local_tifffile=import_local_tifffile)
-    nt.assert_equal(s.data.shape, (2, 68, 68))
-    nt.assert_equal(s.axes_manager[0].units, t.Undefined)
-    nt.assert_equal(s.axes_manager[1].units, t.Undefined)
-    nt.assert_equal(s.axes_manager[2].units, t.Undefined)
-    nt.assert_almost_equal(s.axes_manager[0].scale, 1.0, places=5)
-    nt.assert_almost_equal(s.axes_manager[1].scale, 1.0, places=5)
-    nt.assert_almost_equal(s.axes_manager[2].scale, 1.0, places=5)
+    assert s.data.shape == (2, 68, 68)
+    assert s.axes_manager[0].units == t.Undefined
+    assert s.axes_manager[1].units == t.Undefined
+    assert s.axes_manager[2].units == t.Undefined
+    assert_allclose(s.axes_manager[0].scale, 1.0, atol=1E-5)
+    assert_allclose(s.axes_manager[1].scale, 1.0, atol=1E-5)
+    assert_allclose(s.axes_manager[2].scale, 1.0, atol=1E-5)
 
 
 def test_read_unit_from_imagej_no_scale(import_local_tifffile=False):
     fname = os.path.join(MY_PATH, 'tiff_files',
                          'test_loading_image_saved_with_imageJ_no_scale.tif')
     s = hs.load(fname, import_local_tifffile=import_local_tifffile)
-    nt.assert_equal(s.axes_manager[0].units, t.Undefined)
-    nt.assert_equal(s.axes_manager[1].units, t.Undefined)
-    nt.assert_almost_equal(s.axes_manager[0].scale, 1.0, places=5)
-    nt.assert_almost_equal(s.axes_manager[1].scale, 1.0, places=5)
+    assert s.axes_manager[0].units == t.Undefined
+    assert s.axes_manager[1].units == t.Undefined
+    assert_allclose(s.axes_manager[0].scale, 1.0, atol=1E-5)
+    assert_allclose(s.axes_manager[1].scale, 1.0, atol=1E-5)
 
 
 def test_write_read_unit_imagej(import_local_tifffile=True):
@@ -173,9 +174,9 @@ def test_write_read_unit_imagej(import_local_tifffile=True):
             tmpdir, 'test_loading_image_saved_with_imageJ2.tif')
         s.save(fname2, export_scale=True, overwrite=True)
         s2 = hs.load(fname2, import_local_tifffile=import_local_tifffile)
-        nt.assert_equal(s2.axes_manager[0].units, 'micron')
-        nt.assert_equal(s2.axes_manager[1].units, 'micron')
-        nt.assert_equal(s.data.shape, s.data.shape)
+        assert s2.axes_manager[0].units == 'micron'
+        assert s2.axes_manager[1].units == 'micron'
+        assert s.data.shape == s.data.shape
 
 
 def test_write_read_unit_imagej_with_description(import_local_tifffile=True):
@@ -184,24 +185,24 @@ def test_write_read_unit_imagej_with_description(import_local_tifffile=True):
     s = hs.load(fname, import_local_tifffile=import_local_tifffile)
     s.axes_manager[0].units = 'µm'
     s.axes_manager[1].units = 'µm'
-    nt.assert_almost_equal(s.axes_manager[0].scale, 0.16867, places=5)
-    nt.assert_almost_equal(s.axes_manager[1].scale, 0.16867, places=5)
+    assert_allclose(s.axes_manager[0].scale, 0.16867, atol=1E-5)
+    assert_allclose(s.axes_manager[1].scale, 0.16867, atol=1E-5)
     with tempfile.TemporaryDirectory() as tmpdir:
         fname2 = os.path.join(tmpdir, 'description.tif')
         s.save(fname2, export_scale=False, overwrite=True, description='test')
         s2 = hs.load(fname2, import_local_tifffile=import_local_tifffile)
-        nt.assert_equal(s2.axes_manager[0].units, t.Undefined)
-        nt.assert_equal(s2.axes_manager[1].units, t.Undefined)
-        nt.assert_almost_equal(s2.axes_manager[0].scale, 1.0, places=5)
-        nt.assert_almost_equal(s2.axes_manager[1].scale, 1.0, places=5)
+        assert s2.axes_manager[0].units == t.Undefined
+        assert s2.axes_manager[1].units == t.Undefined
+        assert_allclose(s2.axes_manager[0].scale, 1.0, atol=1E-5)
+        assert_allclose(s2.axes_manager[1].scale, 1.0, atol=1E-5)
 
         fname3 = os.path.join(tmpdir, 'description2.tif')
         s.save(fname3, export_scale=True, overwrite=True, description='test')
         s3 = hs.load(fname3, import_local_tifffile=import_local_tifffile)
-        nt.assert_equal(s3.axes_manager[0].units, 'micron')
-        nt.assert_equal(s3.axes_manager[1].units, 'micron')
-        nt.assert_almost_equal(s3.axes_manager[0].scale, 0.16867, places=5)
-        nt.assert_almost_equal(s3.axes_manager[1].scale, 0.16867, places=5)
+        assert s3.axes_manager[0].units == 'micron'
+        assert s3.axes_manager[1].units == 'micron'
+        assert_allclose(s3.axes_manager[0].scale, 0.16867, atol=1E-5)
+        assert_allclose(s3.axes_manager[1].scale, 0.16867, atol=1E-5)
 
 
 def test_saving_with_custom_tag():
@@ -216,8 +217,8 @@ def test_saving_with_custom_tag():
         extratag = [(65000, 's', 1, "Random metadata", False)]
         s.save(fname, extratags=extratag, overwrite=True)
         s2 = hs.load(fname)
-        nt.assert_equal(s2.original_metadata['Number_65000'],
-                        b"Random metadata")
+        assert (s2.original_metadata['Number_65000'] ==
+                b"Random metadata")
 
 
 def test_read_unit_from_dm():
@@ -233,27 +234,27 @@ def test_read_unit_from_dm_local_tifffile():
 def _test_read_unit_from_dm(import_local_tifffile=False):
     fname = os.path.join(MY_PATH2, 'test_loading_image_saved_with_DM.tif')
     s = hs.load(fname, import_local_tifffile=import_local_tifffile)
-    nt.assert_equal(s.axes_manager[0].units, 'µm')
-    nt.assert_equal(s.axes_manager[1].units, 'µm')
-    nt.assert_almost_equal(s.axes_manager[0].scale, 0.16867, places=5)
-    nt.assert_almost_equal(s.axes_manager[1].scale, 0.16867, places=5)
-    nt.assert_almost_equal(s.axes_manager[0].offset, 139.66264, places=5)
-    nt.assert_almost_equal(s.axes_manager[1].offset, 128.19276, places=5)
+    assert s.axes_manager[0].units == 'µm'
+    assert s.axes_manager[1].units == 'µm'
+    assert_allclose(s.axes_manager[0].scale, 0.16867, atol=1E-5)
+    assert_allclose(s.axes_manager[1].scale, 0.16867, atol=1E-5)
+    assert_allclose(s.axes_manager[0].offset, 139.66264, atol=1E-5)
+    assert_allclose(s.axes_manager[1].offset, 128.19276, atol=1E-5)
     with tempfile.TemporaryDirectory() as tmpdir:
         fname2 = os.path.join(tmpdir, "DM2.tif")
         s.save(fname2, overwrite=True)
         s2 = hs.load(fname2)
         _compare_signal_shape_data(s, s2)
-        nt.assert_equal(s2.axes_manager[0].units, 'micron')
-        nt.assert_equal(s2.axes_manager[1].units, 'micron')
-        nt.assert_almost_equal(s2.axes_manager[0].scale, s.axes_manager[0].scale,
-                               places=5)
-        nt.assert_almost_equal(s2.axes_manager[1].scale, s.axes_manager[1].scale,
-                               places=5)
-        nt.assert_almost_equal(s2.axes_manager[0].offset, s.axes_manager[0].offset,
-                               places=5)
-        nt.assert_almost_equal(s2.axes_manager[1].offset, s.axes_manager[1].offset,
-                               places=5)
+        assert s2.axes_manager[0].units == 'micron'
+        assert s2.axes_manager[1].units == 'micron'
+        assert_allclose(s2.axes_manager[0].scale, s.axes_manager[0].scale,
+                        atol=1E-5)
+        assert_allclose(s2.axes_manager[1].scale, s.axes_manager[1].scale,
+                        atol=1E-5)
+        assert_allclose(s2.axes_manager[0].offset, s.axes_manager[0].offset,
+                        atol=1E-5)
+        assert_allclose(s2.axes_manager[1].offset, s.axes_manager[1].offset,
+                        atol=1E-5)
 
 
 def test_write_scale_unit():
@@ -376,15 +377,15 @@ def test_write_scale_unit_image_stack():
         s.save(fname, overwrite=True, export_scale=True)
         s1 = hs.load(fname)
         _compare_signal_shape_data(s, s1)
-        nt.assert_equal(s1.axes_manager[0].units, 'nm')
+        assert s1.axes_manager[0].units == 'nm'
         # only one unit can be read
-        nt.assert_equal(s1.axes_manager[1].units, 'mm')
-        nt.assert_equal(s1.axes_manager[2].units, 'mm')
-        nt.assert_almost_equal(
+        assert s1.axes_manager[1].units == 'mm'
+        assert s1.axes_manager[2].units == 'mm'
+        assert_allclose(
             s1.axes_manager[0].scale, s.axes_manager[0].scale)
-        nt.assert_almost_equal(
+        assert_allclose(
             s1.axes_manager[1].scale, s.axes_manager[1].scale)
-        nt.assert_almost_equal(
+        assert_allclose(
             s1.axes_manager[2].scale, s.axes_manager[2].scale)
 
 
@@ -400,83 +401,83 @@ def test_saving_loading_stack_no_scale():
 def test_read_FEI_SEM_scale_metadata_8bits():
     fname = os.path.join(MY_PATH2, 'FEI-Helios-Ebeam-8bits.tif')
     s = hs.load(fname)
-    nt.assert_equal(s.axes_manager[0].units, 'm')
-    nt.assert_equal(s.axes_manager[1].units, 'm')
-    nt.assert_almost_equal(s.axes_manager[0].scale, 3.3724e-06, places=12)
-    nt.assert_almost_equal(s.axes_manager[1].scale, 3.3724e-06, places=12)
-    nt.assert_equal(s.data.dtype, 'uint8')
+    assert s.axes_manager[0].units == 'm'
+    assert s.axes_manager[1].units == 'm'
+    assert_allclose(s.axes_manager[0].scale, 3.3724e-06, atol=1E-12)
+    assert_allclose(s.axes_manager[1].scale, 3.3724e-06, atol=1E-12)
+    assert s.data.dtype == 'uint8'
 
 
 def test_read_FEI_SEM_scale_metadata_16bits():
     fname = os.path.join(MY_PATH2, 'FEI-Helios-Ebeam-16bits.tif')
     s = hs.load(fname)
-    nt.assert_equal(s.axes_manager[0].units, 'm')
-    nt.assert_equal(s.axes_manager[1].units, 'm')
-    nt.assert_almost_equal(s.axes_manager[0].scale, 3.3724e-06, places=12)
-    nt.assert_almost_equal(s.axes_manager[1].scale, 3.3724e-06, places=12)
-    nt.assert_equal(s.data.dtype, 'uint16')
+    assert s.axes_manager[0].units == 'm'
+    assert s.axes_manager[1].units == 'm'
+    assert_allclose(s.axes_manager[0].scale, 3.3724e-06, atol=1E-12)
+    assert_allclose(s.axes_manager[1].scale, 3.3724e-06, atol=1E-12)
+    assert s.data.dtype == 'uint16'
 
 
 def test_read_Zeiss_SEM_scale_metadata_1k_image():
     fname = os.path.join(MY_PATH2, 'test_tiff_Zeiss_SEM_1k.tif')
     s = hs.load(fname)
-    nt.assert_equal(s.axes_manager[0].units, 'm')
-    nt.assert_equal(s.axes_manager[1].units, 'm')
-    nt.assert_almost_equal(s.axes_manager[0].scale, 2.614514e-06, places=12)
-    nt.assert_almost_equal(s.axes_manager[1].scale, 2.614514e-06, places=12)
-    nt.assert_equal(s.data.dtype, 'uint16')
+    assert s.axes_manager[0].units == 'm'
+    assert s.axes_manager[1].units == 'm'
+    assert_allclose(s.axes_manager[0].scale, 2.614514e-06, atol=1E-12)
+    assert_allclose(s.axes_manager[1].scale, 2.614514e-06, atol=1E-12)
+    assert s.data.dtype == 'uint16'
 
 
 def test_read_Zeiss_SEM_scale_metadata_512_image():
     fname = os.path.join(MY_PATH2, 'test_tiff_Zeiss_SEM_512.tif')
     s = hs.load(fname)
-    nt.assert_equal(s.axes_manager[0].units, 'm')
-    nt.assert_equal(s.axes_manager[1].units, 'm')
-    nt.assert_almost_equal(s.axes_manager[0].scale, 7.4240e-08, places=12)
-    nt.assert_almost_equal(s.axes_manager[1].scale, 7.4240e-08, places=12)
-    nt.assert_equal(s.data.dtype, 'uint16')
+    assert s.axes_manager[0].units == 'm'
+    assert s.axes_manager[1].units == 'm'
+    assert_allclose(s.axes_manager[0].scale, 7.4240e-08, atol=1E-12)
+    assert_allclose(s.axes_manager[1].scale, 7.4240e-08, atol=1E-12)
+    assert s.data.dtype == 'uint16'
 
 
 def test_read_RGB_Zeiss_optical_scale_metadata():
     fname = os.path.join(MY_PATH2, 'optical_Zeiss_AxioVision_RGB.tif')
     s = hs.load(fname, import_local_tifffile=True)
     dtype = np.dtype([('R', 'u1'), ('G', 'u1'), ('B', 'u1')])
-    nt.assert_equal(s.data.dtype, dtype)
-    nt.assert_equal(s.data.shape, (10, 13))
-    nt.assert_equal(s.axes_manager[0].units, t.Undefined)
-    nt.assert_equal(s.axes_manager[1].units, t.Undefined)
-    nt.assert_almost_equal(s.axes_manager[0].scale, 1.0, places=3)
-    nt.assert_almost_equal(s.axes_manager[1].scale, 1.0, places=3)
+    assert s.data.dtype == dtype
+    assert s.data.shape == (10, 13)
+    assert s.axes_manager[0].units == t.Undefined
+    assert s.axes_manager[1].units == t.Undefined
+    assert_allclose(s.axes_manager[0].scale, 1.0, atol=1E-3)
+    assert_allclose(s.axes_manager[1].scale, 1.0, atol=1E-3)
 
 
 def test_read_BW_Zeiss_optical_scale_metadata():
     fname = os.path.join(MY_PATH2, 'optical_Zeiss_AxioVision_BW.tif')
     s = hs.load(fname, force_read_resolution=True, import_local_tifffile=True)
-    nt.assert_equal(s.data.dtype, np.uint16)
-    nt.assert_equal(s.data.shape, (10, 13))
-    nt.assert_equal(s.axes_manager[0].units, 'µm')
-    nt.assert_equal(s.axes_manager[1].units, 'µm')
-    nt.assert_almost_equal(s.axes_manager[0].scale, 169.3333, places=3)
-    nt.assert_almost_equal(s.axes_manager[1].scale, 169.3333, places=3)
+    assert s.data.dtype == np.uint16
+    assert s.data.shape == (10, 13)
+    assert s.axes_manager[0].units == 'µm'
+    assert s.axes_manager[1].units == 'µm'
+    assert_allclose(s.axes_manager[0].scale, 169.3333, atol=1E-3)
+    assert_allclose(s.axes_manager[1].scale, 169.3333, atol=1E-3)
 
 
 def test_read_BW_Zeiss_optical_scale_metadata2():
     fname = os.path.join(MY_PATH2, 'optical_Zeiss_AxioVision_BW.tif')
     s = hs.load(fname, force_read_resolution=True)
-    nt.assert_equal(s.data.dtype, np.uint16)
-    nt.assert_equal(s.data.shape, (10, 13))
-    nt.assert_equal(s.axes_manager[0].units, 'µm')
-    nt.assert_equal(s.axes_manager[1].units, 'µm')
-    nt.assert_almost_equal(s.axes_manager[0].scale, 169.3333, places=3)
-    nt.assert_almost_equal(s.axes_manager[1].scale, 169.3333, places=3)
+    assert s.data.dtype == np.uint16
+    assert s.data.shape == (10, 13)
+    assert s.axes_manager[0].units == 'µm'
+    assert s.axes_manager[1].units == 'µm'
+    assert_allclose(s.axes_manager[0].scale, 169.3333, atol=1E-3)
+    assert_allclose(s.axes_manager[1].scale, 169.3333, atol=1E-3)
 
 
 def test_read_BW_Zeiss_optical_scale_metadata3():
     fname = os.path.join(MY_PATH2, 'optical_Zeiss_AxioVision_BW.tif')
     s = hs.load(fname, force_read_resolution=False)
-    nt.assert_equal(s.data.dtype, np.uint16)
-    nt.assert_equal(s.data.shape, (10, 13))
-    nt.assert_equal(s.axes_manager[0].units, t.Undefined)
-    nt.assert_equal(s.axes_manager[1].units, t.Undefined)
-    nt.assert_almost_equal(s.axes_manager[0].scale, 1.0, places=3)
-    nt.assert_almost_equal(s.axes_manager[1].scale, 1.0, places=3)
+    assert s.data.dtype == np.uint16
+    assert s.data.shape == (10, 13)
+    assert s.axes_manager[0].units == t.Undefined
+    assert s.axes_manager[1].units == t.Undefined
+    assert_allclose(s.axes_manager[0].scale, 1.0, atol=1E-3)
+    assert_allclose(s.axes_manager[1].scale, 1.0, atol=1E-3)
