@@ -18,22 +18,18 @@
 from unittest import mock
 
 import numpy as np
-import nose.tools as nt
-from nose.plugins.skip import SkipTest
 from scipy.signal import savgol_filter
-try:
-    from statsmodels.nonparametric.smoothers_lowess import lowess
-    skip_lowess = False
-except:
-    skip_lowess = True
+import pytest
 
 from hyperspy.misc.tv_denoise import _tv_denoise_1d
+from hyperspy.decorators import lazifyTestClass
 import hyperspy.api as hs
 
 
+@lazifyTestClass
 class TestAlignTools:
 
-    def setUp(self):
+    def setup_method(self, method):
         s = hs.signals.Signal1D(np.zeros((10, 100)))
         self.scale = 0.1
         self.offset = -2
@@ -62,32 +58,32 @@ class TestAlignTools:
                   1 *
                   self.ishifts[:, np.newaxis] *
                   self.scale, show_progressbar=None)
-        nt.assert_true(m.data_changed.called)
+        assert m.data_changed.called
         i_zlp = s.axes_manager.signal_axes[0].value2index(0)
-        nt.assert_true(np.allclose(s.data[:, i_zlp], 12))
+        assert np.allclose(s.data[:, i_zlp], 12)
         # Check that at the edges of the spectrum the value == to the
         # background value. If it wasn't it'll mean that the cropping
         # code is buggy
-        nt.assert_true((s.data[:, -1] == 2).all())
-        nt.assert_true((s.data[:, 0] == 2).all())
+        assert (s.data[:, -1] == 2).all()
+        assert (s.data[:, 0] == 2).all()
         # Check that the calibration is correct
-        nt.assert_equal(s.axes_manager._axes[1].offset, self.new_offset)
-        nt.assert_equal(s.axes_manager._axes[1].scale, self.scale)
+        assert s.axes_manager._axes[1].offset == self.new_offset
+        assert s.axes_manager._axes[1].scale == self.scale
 
     def test_align(self):
         s = self.signal
         s.align1D(show_progressbar=None)
         i_zlp = s.axes_manager.signal_axes[0].value2index(0)
-        nt.assert_true(np.allclose(s.data[:, i_zlp], 12))
+        assert np.allclose(s.data[:, i_zlp], 12)
         # Check that at the edges of the spectrum the value == to the
         # background value. If it wasn't it'll mean that the cropping
         # code is buggy
-        nt.assert_true((s.data[:, -1] == 2).all())
-        nt.assert_true((s.data[:, 0] == 2).all())
+        assert (s.data[:, -1] == 2).all()
+        assert (s.data[:, 0] == 2).all()
         # Check that the calibration is correct
-        nt.assert_equal(
-            s.axes_manager._axes[1].offset, self.new_offset)
-        nt.assert_equal(s.axes_manager._axes[1].scale, self.scale)
+        assert (
+            s.axes_manager._axes[1].offset == self.new_offset)
+        assert s.axes_manager._axes[1].scale == self.scale
 
     def test_align_expand(self):
         s = self.signal
@@ -98,41 +94,43 @@ class TestAlignTools:
         Nnan_data = np.sum(np.isnan(s.data), axis=1)
         # Due to interpolation, the number of NaNs in the data might
         # be 2 higher (left and right side) than expected
-        nt.assert_true(np.all(Nnan_data - Nnan <= 2))
+        assert np.all(Nnan_data - Nnan <= 2)
 
         # Check actual alignment of zlp
         i_zlp = s.axes_manager.signal_axes[0].value2index(0)
-        nt.assert_true(np.allclose(s.data[:, i_zlp], 12))
+        assert np.allclose(s.data[:, i_zlp], 12)
 
 
+@lazifyTestClass
 class TestShift1D:
 
-    def setUp(self):
+    def setup_method(self, method):
         self.s = hs.signals.Signal1D(np.arange(10))
         self.s.axes_manager[0].scale = 0.2
 
     def test_crop_left(self):
         s = self.s
         s.shift1D(np.array((0.01)), crop=True, show_progressbar=None)
-        nt.assert_equal(
+        assert (
             tuple(
-                s.axes_manager[0].axis), tuple(
+                s.axes_manager[0].axis) == tuple(
                 np.arange(
                     0.2, 2., 0.2)))
 
     def test_crop_right(self):
         s = self.s
         s.shift1D(np.array((-0.01)), crop=True, show_progressbar=None)
-        nt.assert_equal(
+        assert (
             tuple(
-                s.axes_manager[0].axis), tuple(
+                s.axes_manager[0].axis) == tuple(
                 np.arange(
                     0., 1.8, 0.2)))
 
 
+@lazifyTestClass
 class TestFindPeaks1D:
 
-    def setUp(self):
+    def setup_method(self, method):
         x = np.arange(0, 50, 0.01)
         s = hs.signals.Signal1D(np.vstack((np.cos(x), np.sin(x))))
         s.axes_manager.signal_axes[0].scale = 0.01
@@ -141,40 +139,50 @@ class TestFindPeaks1D:
         self.signal = s
 
     def test_single_spectrum(self):
-        peaks = self.signal.inav[0].find_peaks1D_ohaver()
-        nt.assert_true(np.allclose(
-            peaks[0]['position'], self.peak_positions0, rtol=1e-5, atol=1e-4))
+        peaks = self.signal.inav[0].find_peaks1D_ohaver()[0]
+        if self.signal._lazy:
+            peaks = peaks.compute()
+        assert np.allclose(
+            peaks['position'], self.peak_positions0, rtol=1e-5, atol=1e-4)
 
     def test_two_spectra(self):
-        peaks = self.signal.find_peaks1D_ohaver()
-        nt.assert_true(np.allclose(
-            peaks[1]['position'], self.peak_positions1, rtol=1e-5, atol=1e-4))
+        peaks = self.signal.find_peaks1D_ohaver()[1]
+        if self.signal._lazy:
+            peaks = peaks.compute()
+        assert np.allclose(
+            peaks['position'], self.peak_positions1, rtol=1e-5, atol=1e-4)
 
     def test_height(self):
-        peaks = self.signal.find_peaks1D_ohaver()
-        nt.assert_true(np.allclose(
-            peaks[1]['height'], 1.0, rtol=1e-5, atol=1e-4))
+        peaks = self.signal.find_peaks1D_ohaver()[1]
+        if self.signal._lazy:
+            peaks = peaks.compute()
+        assert np.allclose(
+            peaks['height'], 1.0, rtol=1e-5, atol=1e-4)
 
     def test_width(self):
-        peaks = self.signal.find_peaks1D_ohaver()
-        nt.assert_true(np.allclose(
-            peaks[1]['width'], 3.5758, rtol=1e-4, atol=1e-4),
-            msg="One or several widths are not close enough to expected " +
-            "value (3.5758): " + str(peaks[1]['width']))
+        peaks = self.signal.find_peaks1D_ohaver()[1]
+        if self.signal._lazy:
+            peaks = peaks.compute()
+        assert np.allclose(peaks['width'], 3.5758, rtol=1e-4, atol=1e-4)
 
     def test_n_peaks(self):
-        peaks = self.signal.find_peaks1D_ohaver()
-        nt.assert_equal(len(peaks[1]), 8)
+        peaks = self.signal.find_peaks1D_ohaver()[1]
+        if self.signal._lazy:
+            peaks = peaks.compute()
+        assert len(peaks) == 8
 
     def test_maxpeaksn(self):
         for n in range(1, 10):
-            peaks = self.signal.find_peaks1D_ohaver(maxpeakn=n)
-            nt.assert_equal(len(peaks[1]), min((8, n)))
+            peaks = self.signal.find_peaks1D_ohaver(maxpeakn=n)[1]
+            if self.signal._lazy:
+                peaks = peaks.compute()
+            assert len(peaks) == min((8, n))
 
 
+@lazifyTestClass
 class TestInterpolateInBetween:
 
-    def setUp(self):
+    def setup_method(self, method):
         s = hs.signals.Signal1D(np.arange(40).reshape((2, 20)))
         s.axes_manager.signal_axes[0].scale = 0.1
         s.isig[8:12] = 0
@@ -186,7 +194,7 @@ class TestInterpolateInBetween:
         s.events.data_changed.connect(m.data_changed)
         s.interpolate_in_between(8, 12, show_progressbar=None)
         np.testing.assert_array_equal(s.data, np.arange(20))
-        nt.assert_true(m.data_changed.called)
+        assert m.data_changed.called
 
     def test_single_spectrum_in_units(self):
         s = self.s.inav[0]
@@ -201,7 +209,9 @@ class TestInterpolateInBetween:
     def test_delta_int(self):
         s = self.s.inav[0]
         s.change_dtype('float')
-        s.data[12] *= 10
+        tmp = np.zeros_like(s.data)
+        tmp[12] = s.data[12]
+        s.data += tmp * 9.
         s.interpolate_in_between(8, 12, delta=2, kind='cubic')
         print(s.data[8:12])
         np.testing.assert_allclose(
@@ -210,7 +220,9 @@ class TestInterpolateInBetween:
     def test_delta_float(self):
         s = self.s.inav[0]
         s.change_dtype('float')
-        s.data[12] *= 10.
+        tmp = np.zeros_like(s.data)
+        tmp[12] = s.data[12]
+        s.data += tmp * 9.
         s.interpolate_in_between(8, 12, delta=0.31, kind='cubic')
         print(s.data[8:12])
         np.testing.assert_allclose(
@@ -218,65 +230,80 @@ class TestInterpolateInBetween:
                                     155.48258721, 170.33564422]))
 
 
+@lazifyTestClass
 class TestEstimatePeakWidth:
 
-    def setUp(self):
+    def setup_method(self, method):
         scale = 0.1
         window = 2
         x = np.arange(-window, window, scale)
-        g = hs.model.components1D.Gaussian()
+        g = hs.model.components1D.Gaussian(sigma=0.3)
         s = hs.signals.Signal1D(g.function(x))
         s.axes_manager[-1].scale = scale
         self.s = s
+        self.rtol = 1e-7
+        self.atol = 0
 
     def test_full_range(self):
         width, left, right = self.s.estimate_peak_width(
             window=None,
             return_interval=True,
             show_progressbar=None)
-        nt.assert_equal(width, 2.35482074)
-        nt.assert_equal(left, 0.82258963)
-        nt.assert_equal(right, 3.17741037)
+        np.testing.assert_allclose(width.data, 0.7065102,
+                                   rtol=self.rtol, atol=self.atol)
+        np.testing.assert_allclose(left.data, 1.6467449,
+                                   rtol=self.rtol, atol=self.atol)
+        np.testing.assert_allclose(right.data, 2.353255,
+                                   rtol=self.rtol, atol=self.atol)
         for t in (width, left, right):
-            nt.assert_equal(t.metadata.Signal.signal_type, "")
-            nt.assert_equal(t.axes_manager.signal_dimension, 0)
+            assert t.metadata.Signal.signal_type == ""
+            assert t.axes_manager.signal_dimension == 0
 
     def test_too_narrow_range(self):
         width, left, right = self.s.estimate_peak_width(
-            window=2.2,
+            window=0.5,
             return_interval=True,
             show_progressbar=None)
-        nt.assert_equal(width, np.nan)
-        nt.assert_equal(left, np.nan)
-        nt.assert_equal(right, np.nan)
+        assert np.isnan(width.data).all()
+        assert np.isnan(left.data).all()
+        assert np.isnan(right.data).all()
 
     def test_two_peaks(self):
         s = self.s.deepcopy()
-        s.shift1D(np.array([0.5]), show_progressbar=None)
-        self.s.isig[:-5] += s
+        s.shift1D(np.array([1.0]), show_progressbar=None)
+        self.s = self.s.isig[10:] + s
         width, left, right = self.s.estimate_peak_width(
             window=None,
             return_interval=True,
             show_progressbar=None)
-        nt.assert_equal(width, np.nan)
-        nt.assert_equal(left, np.nan)
-        nt.assert_equal(right, np.nan)
+        assert np.isnan(width.data).all()
+        assert np.isnan(left.data).all()
+        assert np.isnan(right.data).all()
 
 
+@lazifyTestClass(rtol=1e-4, atol=0.4)
 class TestSmoothing:
 
-    def setUp(self):
+    def setup_method(self, method):
         n, m = 2, 100
-        self.s = hs.signals.Signal1D(np.arange(n * m).reshape(n, m))
+        self.s = hs.signals.Signal1D(
+            np.arange(
+                n * m,
+                dtype='float').reshape(
+                n,
+                m))
         np.random.seed(1)
         self.s.add_gaussian_noise(0.1)
+        self.rtol = 1e-7
+        self.atol = 0
 
-    def test_lowess(self):
-        if skip_lowess:
-            raise SkipTest
+    @pytest.mark.parametrize('parallel', [pytest.mark.parallel(True), False])
+    def test_lowess(self, parallel):
+        pytest.importorskip("statsmodels")
+        from statsmodels.nonparametric.smoothers_lowess import lowess
         frac = 0.5
         it = 1
-        data = self.s.data.copy()
+        data = np.asanyarray(self.s.data, dtype='float')
         for i in range(data.shape[0]):
             data[i, :] = lowess(
                 endog=data[i, :],
@@ -287,19 +314,24 @@ class TestSmoothing:
                 return_sorted=False,)
         self.s.smooth_lowess(smoothing_parameter=frac,
                              number_of_iterations=it,
-                             show_progressbar=None)
-        nt.assert_true(np.allclose(data, self.s.data))
+                             show_progressbar=None,
+                             parallel=parallel)
+        np.testing.assert_allclose(self.s.data, data,
+                                   rtol=self.rtol, atol=self.atol)
 
-    def test_tv(self):
+    @pytest.mark.parametrize('parallel', [pytest.mark.parallel(True), False])
+    def test_tv(self, parallel):
         weight = 1
-        data = self.s.data.copy()
+        data = np.asanyarray(self.s.data, dtype='float')
         for i in range(data.shape[0]):
             data[i, :] = _tv_denoise_1d(
                 im=data[i, :],
                 weight=weight,)
         self.s.smooth_tv(smoothing_parameter=weight,
-                         show_progressbar=None)
-        nt.assert_true(np.allclose(data, self.s.data))
+                         show_progressbar=None,
+                         parallel=parallel)
+        np.testing.assert_allclose(data, self.s.data,
+                                   rtol=self.rtol, atol=self.atol)
 
     def test_savgol(self):
         window_length = 13
@@ -316,4 +348,4 @@ class TestSmoothing:
             window_length=window_length,
             polynomial_order=polyorder,
             differential_order=deriv,)
-        nt.assert_true(np.allclose(data, self.s.data))
+        np.testing.assert_allclose(data, self.s.data)
