@@ -18,18 +18,21 @@
 import sys
 
 import numpy as np
-import nose.tools as nt
+
+from numpy.testing import assert_allclose
 
 from hyperspy.signals import EDSSEMSpectrum
 from hyperspy.defaults_parser import preferences
 from hyperspy.components1d import Gaussian
 from hyperspy import utils
 from hyperspy.misc.test_utils import assert_warns
+from hyperspy.decorators import lazifyTestClass
 
 
+@lazifyTestClass
 class Test_metadata:
 
-    def setUp(self):
+    def setup_method(self, method):
         # Create an empty spectrum
         s = EDSSEMSpectrum(np.ones((4, 2, 1024)))
         s.axes_manager.signal_axes[0].scale = 1e-3
@@ -46,28 +49,26 @@ class Test_metadata:
         s = self.signal
         old_metadata = s.metadata.deepcopy()
         sSum = s.sum(0)
-        nt.assert_equal(
-            sSum.metadata.Acquisition_instrument.SEM.Detector.EDS.live_time,
+        assert (
+            sSum.metadata.Acquisition_instrument.SEM.Detector.EDS.live_time ==
             3.1 * 2)
         # Check that metadata is unchanged
         print(old_metadata, s.metadata)      # Capture for comparison on error
-        nt.assert_dict_equal(old_metadata.as_dictionary(),
-                             s.metadata.as_dictionary(),
-                             "Source metadata changed")
+        assert (old_metadata.as_dictionary() ==
+                s.metadata.as_dictionary()), "Source metadata changed"
 
     def test_sum_live_time2(self):
         s = self.signal
         old_metadata = s.metadata.deepcopy()
         sSum = s.sum((0, 1))
-        nt.assert_equal(
-            sSum.metadata.Acquisition_instrument.SEM.Detector.EDS.live_time,
+        assert (
+            sSum.metadata.Acquisition_instrument.SEM.Detector.EDS.live_time ==
             3.1 *
             2 * 4)
         # Check that metadata is unchanged
         print(old_metadata, s.metadata)      # Capture for comparison on error
-        nt.assert_dict_equal(old_metadata.as_dictionary(),
-                             s.metadata.as_dictionary(),
-                             "Source metadata changed")
+        assert (old_metadata.as_dictionary() ==
+                s.metadata.as_dictionary()), "Source metadata changed"
 
     def test_sum_live_time_out_arg(self):
         s = self.signal
@@ -75,9 +76,9 @@ class Test_metadata:
         s.metadata.Acquisition_instrument.SEM.Detector.EDS.live_time = 4.2
         s_resum = s.sum(0)
         r = s.sum(0, out=sSum)
-        nt.assert_is_none(r)
-        nt.assert_equal(
-            s_resum.metadata.Acquisition_instrument.SEM.Detector.EDS.live_time,
+        assert r is None
+        assert (
+            s_resum.metadata.Acquisition_instrument.SEM.Detector.EDS.live_time ==
             sSum.metadata.Acquisition_instrument.SEM.Detector.EDS.live_time)
         np.testing.assert_allclose(s_resum.data, sSum.data)
 
@@ -86,76 +87,75 @@ class Test_metadata:
         old_metadata = s.metadata.deepcopy()
         dim = s.axes_manager.shape
         s = s.rebin([dim[0] / 2, dim[1] / 2, dim[2]])
-        nt.assert_equal(
-            s.metadata.Acquisition_instrument.SEM.Detector.EDS.live_time,
+        assert (
+            s.metadata.Acquisition_instrument.SEM.Detector.EDS.live_time ==
             3.1 *
             2 *
             2)
         # Check that metadata is unchanged
         print(old_metadata, self.signal.metadata)    # Captured on error
-        nt.assert_dict_equal(old_metadata.as_dictionary(),
-                             self.signal.metadata.as_dictionary(),
-                             "Source metadata changed")
+        assert (old_metadata.as_dictionary() ==
+                self.signal.metadata.as_dictionary()), "Source metadata changed"
 
     def test_add_elements(self):
         s = self.signal
         s.add_elements(['Al', 'Ni'])
-        nt.assert_equal(s.metadata.Sample.elements, ['Al', 'Ni'])
+        assert s.metadata.Sample.elements == ['Al', 'Ni']
         s.add_elements(['Al', 'Ni'])
-        nt.assert_equal(s.metadata.Sample.elements, ['Al', 'Ni'])
+        assert s.metadata.Sample.elements == ['Al', 'Ni']
         s.add_elements(["Fe", ])
-        nt.assert_equal(s.metadata.Sample.elements, ['Al', "Fe", 'Ni'])
+        assert s.metadata.Sample.elements == ['Al', "Fe", 'Ni']
         s.set_elements(['Al', 'Ni'])
-        nt.assert_equal(s.metadata.Sample.elements, ['Al', 'Ni'])
+        assert s.metadata.Sample.elements == ['Al', 'Ni']
 
     def test_add_lines(self):
         s = self.signal
         s.add_lines(lines=())
-        nt.assert_equal(s.metadata.Sample.xray_lines, [])
+        assert s.metadata.Sample.xray_lines == []
         s.add_lines(("Fe_Ln",))
-        nt.assert_equal(s.metadata.Sample.xray_lines, ["Fe_Ln"])
+        assert s.metadata.Sample.xray_lines == ["Fe_Ln"]
         s.add_lines(("Fe_Ln",))
-        nt.assert_equal(s.metadata.Sample.xray_lines, ["Fe_Ln"])
+        assert s.metadata.Sample.xray_lines == ["Fe_Ln"]
         s.add_elements(["Ti", ])
         s.add_lines(())
-        nt.assert_equal(
-            s.metadata.Sample.xray_lines, ['Fe_Ln', 'Ti_La'])
+        assert (
+            s.metadata.Sample.xray_lines == ['Fe_Ln', 'Ti_La'])
         s.set_lines((), only_one=False, only_lines=False)
-        nt.assert_equal(s.metadata.Sample.xray_lines,
-                        ['Fe_La', 'Fe_Lb3', 'Fe_Ll', 'Fe_Ln', 'Ti_La',
-                         'Ti_Lb3', 'Ti_Ll', 'Ti_Ln'])
+        assert (s.metadata.Sample.xray_lines ==
+                ['Fe_La', 'Fe_Lb3', 'Fe_Ll', 'Fe_Ln', 'Ti_La',
+                 'Ti_Lb3', 'Ti_Ll', 'Ti_Ln'])
         s.metadata.Acquisition_instrument.SEM.beam_energy = 0.4
         s.set_lines((), only_one=False, only_lines=False)
-        nt.assert_equal(s.metadata.Sample.xray_lines, ['Ti_Ll'])
+        assert s.metadata.Sample.xray_lines == ['Ti_Ll']
 
     def test_add_lines_auto(self):
         s = self.signal
         s.axes_manager.signal_axes[0].scale = 1e-2
         s.set_elements(["Ti", "Al"])
         s.set_lines(['Al_Ka'])
-        nt.assert_equal(
-            s.metadata.Sample.xray_lines, ['Al_Ka', 'Ti_Ka'])
+        assert (
+            s.metadata.Sample.xray_lines == ['Al_Ka', 'Ti_Ka'])
 
         del s.metadata.Sample.xray_lines
         s.set_elements(['Al', 'Ni'])
         s.add_lines()
-        nt.assert_equal(
-            s.metadata.Sample.xray_lines, ['Al_Ka', 'Ni_Ka'])
+        assert (
+            s.metadata.Sample.xray_lines == ['Al_Ka', 'Ni_Ka'])
         s.metadata.Acquisition_instrument.SEM.beam_energy = 10.0
         s.set_lines([])
-        nt.assert_equal(
-            s.metadata.Sample.xray_lines, ['Al_Ka', 'Ni_La'])
+        assert (
+            s.metadata.Sample.xray_lines == ['Al_Ka', 'Ni_La'])
         s.metadata.Acquisition_instrument.SEM.beam_energy = 200
         s.set_elements(['Au', 'Ni'])
         s.set_lines([])
-        nt.assert_equal(s.metadata.Sample.xray_lines,
-                        ['Au_La', 'Ni_Ka'])
+        assert (s.metadata.Sample.xray_lines ==
+                ['Au_La', 'Ni_Ka'])
 
     def test_default_param(self):
         s = self.signal
         mp = s.metadata
-        nt.assert_equal(
-            mp.Acquisition_instrument.SEM.Detector.EDS.energy_resolution_MnKa,
+        assert (
+            mp.Acquisition_instrument.SEM.Detector.EDS.energy_resolution_MnKa ==
             preferences.EDS.eds_mn_ka)
 
     def test_SEM_to_TEM(self):
@@ -174,7 +174,7 @@ class Test_metadata:
             (mpTEM.Acquisition_instrument.TEM.Detector.EDS.
              energy_resolution_MnKa),
             mpTEM.Signal.signal_type]
-        nt.assert_equal(results, resultsTEM)
+        assert results == resultsTEM
 
     def test_get_calibration_from(self):
         s = self.signal
@@ -183,17 +183,18 @@ class Test_metadata:
         energy_axis.scale = 0.01
         energy_axis.offset = -0.10
         s.get_calibration_from(scalib)
-        nt.assert_equal(s.axes_manager.signal_axes[0].scale, energy_axis.scale)
+        assert s.axes_manager.signal_axes[0].scale == energy_axis.scale
 
     def test_take_off_angle(self):
         s = self.signal
-        nt.assert_almost_equal(s.get_take_off_angle(), 12.886929785732487,
-                               places=sys.float_info.dig - 2)
+        assert_allclose(s.get_take_off_angle(), 12.886929785732487,
+                        atol=10**-(sys.float_info.dig - 2))
 
 
+@lazifyTestClass
 class Test_get_lines_intentisity:
 
-    def setUp(self):
+    def setup_method(self, method):
         # Create an empty spectrum
         s = EDSSEMSpectrum(np.zeros((2, 2, 3, 100)))
         energy_axis = s.axes_manager.signal_axes[0]
@@ -213,7 +214,7 @@ class Test_get_lines_intentisity:
         sAl = s.get_lines_intensity(["Al_Ka"],
                                     plot_result=False,
                                     integration_windows=5)[0]
-        nt.assert_equal(sAl.axes_manager.signal_dimension, 0)
+        assert sAl.axes_manager.signal_dimension == 0
         np.testing.assert_allclose(24.99516, sAl.data[0, 0, 0], atol=1e-3)
         sAl = s.inav[0].get_lines_intensity(
             ["Al_Ka"], plot_result=False, integration_windows=5)[0]
@@ -227,9 +228,9 @@ class Test_get_lines_intentisity:
         s.axes_manager[-1].offset = 1.0
         with assert_warns(message="C_Ka is not in the data energy range."):
             sC = s.get_lines_intensity(["C_Ka"], plot_result=False)
-        nt.assert_equal(len(sC), 0)
-        nt.assert_equal(sAl.metadata.Sample.elements, ["Al"])
-        nt.assert_equal(sAl.metadata.Sample.xray_lines, ["Al_Ka"])
+        assert len(sC) == 0
+        assert sAl.metadata.Sample.elements == ["Al"]
+        assert sAl.metadata.Sample.xray_lines == ["Al_Ka"]
 
     def test_eV(self):
         s = self.signal
@@ -245,7 +246,7 @@ class Test_get_lines_intentisity:
     def test_background_substraction(self):
         s = self.signal
         intens = s.get_lines_intensity(["Al_Ka"], plot_result=False)[0].data
-        s += 1.
+        s = s + 1.
         np.testing.assert_allclose(s.estimate_background_windows(
             xray_lines=["Al_Ka"])[0, 0], 1.25666201, atol=1e-3)
         np.testing.assert_allclose(
@@ -271,9 +272,10 @@ class Test_get_lines_intentisity:
             np.array([84163, 89063, 96117, 96700, 99075]))
 
 
+@lazifyTestClass
 class Test_tools_bulk:
 
-    def setUp(self):
+    def setup_method(self, method):
         s = EDSSEMSpectrum(np.ones(1024))
         s.metadata.Acquisition_instrument.SEM.beam_energy = 5.0
         energy_axis = s.axes_manager.signal_axes[0]
@@ -303,9 +305,10 @@ class Test_tools_bulk:
         np.testing.assert_allclose(xr_range, 0.1900368800933955)
 
 
+@lazifyTestClass
 class Test_energy_units:
 
-    def setUp(self):
+    def setup_method(self, method):
         s = EDSSEMSpectrum(np.ones(1024))
         s.metadata.Acquisition_instrument.SEM.beam_energy = 5.0
         s.axes_manager.signal_axes[0].units = 'keV'
@@ -314,16 +317,16 @@ class Test_energy_units:
 
     def test_beam_energy(self):
         s = self.signal
-        nt.assert_equal(s._get_beam_energy(), 5.0)
+        assert s._get_beam_energy() == 5.0
         s.axes_manager.signal_axes[0].units = 'eV'
-        nt.assert_equal(s._get_beam_energy(), 5000.0)
+        assert s._get_beam_energy() == 5000.0
         s.axes_manager.signal_axes[0].units = 'keV'
 
     def test_line_energy(self):
         s = self.signal
-        nt.assert_equal(s._get_line_energy('Al_Ka'), 1.4865)
+        assert s._get_line_energy('Al_Ka') == 1.4865
         s.axes_manager.signal_axes[0].units = 'eV'
-        nt.assert_equal(s._get_line_energy('Al_Ka'), 1486.5)
+        assert s._get_line_energy('Al_Ka') == 1486.5
         s.axes_manager.signal_axes[0].units = 'keV'
 
         np.testing.assert_allclose(s._get_line_energy('Al_Ka', FWHM_MnKa='auto'),
