@@ -17,18 +17,21 @@
 
 
 import numpy as np
-import nose.tools as nt
+import pytest
+from matplotlib.testing.decorators import cleanup
 
 from hyperspy.signals import EDSTEMSpectrum
 from hyperspy.defaults_parser import preferences
 from hyperspy.components1d import Gaussian
 from hyperspy.misc.eds import utils as utils_eds
 from hyperspy.misc.test_utils import ignore_warning
+from hyperspy.decorators import lazifyTestClass
 
 
+@lazifyTestClass
 class Test_metadata:
 
-    def setUp(self):
+    def setup_method(self, method):
         # Create an empty spectrum
         s = EDSTEMSpectrum(np.ones((4, 2, 1024)))
         s.metadata.Acquisition_instrument.TEM.Detector.EDS.live_time = 3.1
@@ -39,27 +42,25 @@ class Test_metadata:
         s = self.signal
         old_metadata = s.metadata.deepcopy()
         sSum = s.sum(0)
-        nt.assert_equal(
-            sSum.metadata.Acquisition_instrument.TEM.Detector.EDS.live_time,
+        assert (
+            sSum.metadata.Acquisition_instrument.TEM.Detector.EDS.live_time ==
             3.1 * 2)
         # Check that metadata is unchanged
         print(old_metadata, s.metadata)      # Capture for comparison on error
-        nt.assert_dict_equal(old_metadata.as_dictionary(),
-                             s.metadata.as_dictionary(),
-                             "Source metadata changed")
+        assert (old_metadata.as_dictionary() ==
+                s.metadata.as_dictionary()), "Source metadata changed"
 
     def test_sum_live_time2(self):
         s = self.signal
         old_metadata = s.metadata.deepcopy()
         sSum = s.sum((0, 1))
-        nt.assert_equal(
-            sSum.metadata.Acquisition_instrument.TEM.Detector.EDS.live_time,
+        assert (
+            sSum.metadata.Acquisition_instrument.TEM.Detector.EDS.live_time ==
             3.1 * 2 * 4)
         # Check that metadata is unchanged
         print(old_metadata, s.metadata)      # Capture for comparison on error
-        nt.assert_dict_equal(old_metadata.as_dictionary(),
-                             s.metadata.as_dictionary(),
-                             "Source metadata changed")
+        assert (old_metadata.as_dictionary() ==
+                s.metadata.as_dictionary()), "Source metadata changed"
 
     def test_sum_live_time_out_arg(self):
         s = self.signal
@@ -67,9 +68,9 @@ class Test_metadata:
         s.metadata.Acquisition_instrument.TEM.Detector.EDS.live_time = 4.2
         s_resum = s.sum(0)
         r = s.sum(0, out=sSum)
-        nt.assert_is_none(r)
-        nt.assert_equal(
-            s_resum.metadata.Acquisition_instrument.TEM.Detector.EDS.live_time,
+        assert r is None
+        assert (
+            s_resum.metadata.Acquisition_instrument.TEM.Detector.EDS.live_time ==
             sSum.metadata.Acquisition_instrument.TEM.Detector.EDS.live_time)
         np.testing.assert_allclose(s_resum.data, sSum.data)
 
@@ -78,31 +79,30 @@ class Test_metadata:
         old_metadata = s.metadata.deepcopy()
         dim = s.axes_manager.shape
         s = s.rebin([dim[0] / 2, dim[1] / 2, dim[2]])
-        nt.assert_equal(
-            s.metadata.Acquisition_instrument.TEM.Detector.EDS.live_time,
+        assert (
+            s.metadata.Acquisition_instrument.TEM.Detector.EDS.live_time ==
             3.1 * 2 * 2)
         # Check that metadata is unchanged
         print(old_metadata, self.signal.metadata)    # Captured on error
-        nt.assert_dict_equal(old_metadata.as_dictionary(),
-                             self.signal.metadata.as_dictionary(),
-                             "Source metadata changed")
+        assert (old_metadata.as_dictionary() ==
+                self.signal.metadata.as_dictionary()), "Source metadata changed"
 
     def test_add_elements(self):
         s = self.signal
         s.add_elements(['Al', 'Ni'])
-        nt.assert_equal(s.metadata.Sample.elements, ['Al', 'Ni'])
+        assert s.metadata.Sample.elements == ['Al', 'Ni']
         s.add_elements(['Al', 'Ni'])
-        nt.assert_equal(s.metadata.Sample.elements, ['Al', 'Ni'])
+        assert s.metadata.Sample.elements == ['Al', 'Ni']
         s.add_elements(["Fe", ])
-        nt.assert_equal(s.metadata.Sample.elements, ['Al', "Fe", 'Ni'])
+        assert s.metadata.Sample.elements == ['Al', "Fe", 'Ni']
         s.set_elements(['Al', 'Ni'])
-        nt.assert_equal(s.metadata.Sample.elements, ['Al', 'Ni'])
+        assert s.metadata.Sample.elements == ['Al', 'Ni']
 
     def test_default_param(self):
         s = self.signal
         mp = s.metadata
-        nt.assert_equal(
-            mp.Acquisition_instrument.TEM.Detector.EDS.energy_resolution_MnKa,
+        assert (
+            mp.Acquisition_instrument.TEM.Detector.EDS.energy_resolution_MnKa ==
             preferences.EDS.eds_mn_ka)
 
     def test_TEM_to_SEM(self):
@@ -119,7 +119,7 @@ class Test_metadata:
         resultsSEM = [
             mpSEM.energy_resolution_MnKa,
             sSEM.metadata.Signal.signal_type]
-        nt.assert_equal(results, resultsSEM)
+        assert results == resultsSEM
 
     def test_get_calibration_from(self):
         s = self.signal
@@ -128,13 +128,14 @@ class Test_metadata:
         energy_axis.scale = 0.01
         energy_axis.offset = -0.10
         s.get_calibration_from(scalib)
-        nt.assert_equal(s.axes_manager.signal_axes[0].scale,
-                        energy_axis.scale)
+        assert (s.axes_manager.signal_axes[0].scale ==
+                energy_axis.scale)
 
 
+@lazifyTestClass
 class Test_quantification:
 
-    def setUp(self):
+    def setup_method(self, method):
         s = EDSTEMSpectrum(np.ones([2, 2, 1024]))
         energy_axis = s.axes_manager.signal_axes[0]
         energy_axis.scale = 1e-2
@@ -237,22 +238,24 @@ class Test_quantification:
         np.testing.assert_allclose(res, [3, 6], atol=1e-3)
 
 
+@lazifyTestClass
 class Test_vacum_mask:
 
-    def setUp(self):
+    def setup_method(self, method):
         s = EDSTEMSpectrum(np.array([np.linspace(0.001, 0.5, 20)] * 100).T)
         s.add_poissonian_noise()
         self.signal = s
 
     def test_vacuum_mask(self):
         s = self.signal
-        nt.assert_true(s.vacuum_mask().data[0])
-        nt.assert_false(s.vacuum_mask().data[-1])
+        assert s.vacuum_mask().data[0]
+        assert not s.vacuum_mask().data[-1]
 
 
+@lazifyTestClass
 class Test_simple_model:
 
-    def setUp(self):
+    def setup_method(self, method):
         s = utils_eds.xray_lines_model(elements=['Al', 'Zn'],
                                        weight_percents=[50, 50])
         self.signal = s
@@ -276,12 +279,11 @@ class Test_simple_model:
             rtol=0.03)
 
 
-class Test_get_lines_intentisity:
-
-    def test_with_signals_examples(self):
-        from hyperspy.misc.example_signals_loading import \
-            load_1D_EDS_TEM_spectrum as EDS_TEM_Spectrum
-        s = EDS_TEM_Spectrum()
+def test_with_signals_examples():
+    from hyperspy.misc.example_signals_loading import \
+        load_1D_EDS_TEM_spectrum as EDS_TEM_Spectrum
+    sig = EDS_TEM_Spectrum()
+    for s in (sig, sig.as_lazy()):
         np.testing.assert_allclose(
             np.array([res.data[0] for res in s.get_lines_intensity()]),
             np.array([3710, 15872]))
@@ -289,33 +291,39 @@ class Test_get_lines_intentisity:
 
 class Test_eds_markers:
 
-    def setUp(self):
+    def setup_method(self, method):
         s = utils_eds.xray_lines_model(elements=['Al', 'Zn'],
                                        weight_percents=[50, 50])
         self.signal = s
 
+    @pytest.mark.skipif("sys.platform == 'darwin'")
+    @cleanup
     def test_plot_auto_add(self):
         s = self.signal
         s.plot(xray_lines=True)
         # Should contain 6 lines
-        nt.assert_sequence_equal(
-            sorted(s._xray_markers.keys()),
+        assert (
+            sorted(s._xray_markers.keys()) ==
             ['Al_Ka', 'Al_Kb', 'Zn_Ka', 'Zn_Kb', 'Zn_La', 'Zn_Lb1'])
 
+    @pytest.mark.skipif("sys.platform == 'darwin'")
+    @cleanup
     def test_manual_add_line(self):
         s = self.signal
         s.add_xray_lines_markers(['Zn_La'])
-        nt.assert_sequence_equal(
-            list(s._xray_markers.keys()),
+        assert (
+            list(s._xray_markers.keys()) ==
             ['Zn_La'])
-        nt.assert_equal(len(s._xray_markers), 1)
+        assert len(s._xray_markers) == 1
         # Check that the line has both a vertical line marker and text marker:
-        nt.assert_equal(len(s._xray_markers['Zn_La']), 2)
+        assert len(s._xray_markers['Zn_La']) == 2
 
+    @pytest.mark.skipif("sys.platform == 'darwin'")
+    @cleanup
     def test_manual_remove_element(self):
         s = self.signal
         s.add_xray_lines_markers(['Zn_Ka', 'Zn_Kb', 'Zn_La'])
         s.remove_xray_lines_markers(['Zn_Kb'])
-        nt.assert_sequence_equal(
-            sorted(s._xray_markers.keys()),
+        assert (
+            sorted(s._xray_markers.keys()) ==
             ['Zn_Ka', 'Zn_La'])
