@@ -1,13 +1,15 @@
-import nose.tools
+
 import numpy as np
 
 from hyperspy._signals.signal1d import Signal1D
 from hyperspy.components1d import Gaussian
+from hyperspy.decorators import lazifyTestClass
 
 
+@lazifyTestClass
 class Test1D:
 
-    def setUp(self):
+    def setup_method(self, method):
         gaussian = Gaussian()
         gaussian.A.value = 20
         gaussian.sigma.value = 10
@@ -18,4 +20,4 @@ class Test1D:
     def test_integrate_in_range(self):
         integrated_signal = self.signal.integrate_in_range(signal_range=(None,
                                                                          None))
-        nose.tools.assert_true(np.allclose(integrated_signal.data, 20,))
+        assert np.allclose(integrated_signal.data, 20,)
